def encrypt(text):
    # This is where the actual ROT13 is applied
    # WHY IS .JOIN NOT WORKING?!
    textlist = list(text)
    newtext = ''
    firsthalf = 'abcdefghijklmABCDEFGHIJKLM'
    lasthalf = 'nopqrstuvwxyzNOPQRSTUVWXYZ'
    for char in textlist:
        if char in firsthalf:
            newtext += lasthalf[firsthalf.index(char)]
        elif char in lasthalf:
            newtext += firsthalf[lasthalf.index(char)]
        else:
            newtext += char

    return newtext

class EncryptHandler(object):
    '''
    This bot allows users to quickly encrypt messages using ROT13 encryption.
    It encrypts/decrypts messages starting with @mention-bot.
    '''

    def usage(self):
        return '''
            This bot uses ROT13 encryption for its purposes.
            It responds to me starting with @mention-bot.
            Feeding encrypted messages into the bot decrypts them.
            '''

    def handle_message(self, message, bot_handler):
        bot_response = self.get_bot_encrypt_response(message)
        bot_handler.send_reply(message, bot_response)

    def get_bot_encrypt_response(self, message):
        original_content = message['content']
        temp_content = encrypt(original_content)
        send_content = "Encrypted/Decrypted text: " + temp_content
        return send_content

handler_class = EncryptHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase
from zulip_bots.lib import StateHandler

class TestVirtualFsBot(BotTestCase):
    bot_name = "virtual_fs"
    help_txt = ('foo_sender@zulip.com:\n\nThis bot implements a virtual file system for a stream.\n'
                'The locations of text are persisted for the lifetime of the bot\n'
                'running, and if you rename a stream, you will lose the info.\n'
                'Example commands:\n\n```\n'
                '@mention-bot sample_conversation: sample conversation with the bot\n'
                '@mention-bot mkdir: create a directory\n'
                '@mention-bot ls: list a directory\n'
                '@mention-bot cd: change directory\n'
                '@mention-bot pwd: show current path\n'
                '@mention-bot write: write text\n'
                '@mention-bot read: read text\n'
                '@mention-bot rm: remove a file\n'
                '@mention-bot rmdir: remove a directory\n'
                '```\n'
                'Use commands like `@mention-bot help write` for more details on specific\ncommands.\n')

    def test_commands_1(self):
        expected = [
            ("cd /home", "foo_sender@zulip.com:\nERROR: invalid path"),
            ("mkdir home", "foo_sender@zulip.com:\ndirectory created"),
            ("pwd", "foo_sender@zulip.com:\n/"),
            ("help", self.help_txt),
            ("help ls", "foo_sender@zulip.com:\nsyntax: ls <optional_path>"),
            ("", self.help_txt),
        ]
        self.check_expected_responses(expected)

    def test_commands_2(self):
        expected = [
            ("help", self.help_txt),
            ("help ls", "foo_sender@zulip.com:\nsyntax: ls <optional_path>"),
            ("", self.help_txt),
            ("pwd", "foo_sender@zulip.com:\n/"),
            ("cd /home", "foo_sender@zulip.com:\nERROR: invalid path"),
            ("mkdir home", "foo_sender@zulip.com:\ndirectory created"),
            ("cd /home", "foo_sender@zulip.com:\nCurrent path: /home/"),
        ]
        self.check_expected_responses(expected)



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

import json

from zulip_bots.test_lib import BotTestCase

class TestYodaBot(BotTestCase):
    bot_name = "yoda"

    def test_bot(self):

        # Test normal sentence (1).
        bot_response = "Learn how to speak like me someday, you will. Yes, hmmm."

        with self.mock_config_info({'api_key': '12345678'}), \
                self.mock_http_conversation('test_1'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'You will learn how to speak like me someday.'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Test normal sentence (2).
        bot_response = "Much to learn, you still have."

        with self.mock_config_info({'api_key': '12345678'}), \
                self.mock_http_conversation('test_2'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'you still have much to learn'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Test only numbers.
        bot_response = "23456.  Herh herh herh."

        with self.mock_config_info({'api_key': '12345678'}), \
                self.mock_http_conversation('test_only_numbers'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': '23456'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Test help.
        bot_response = '''
            This bot allows users to translate a sentence into
            'Yoda speak'.
            Users should preface messages with '@mention-bot'.

            Before running this, make sure to get a Mashape Api token.
            Instructions are in the 'readme.md' file.
            Store it in the 'yoda.conf' file.
            The 'yoda.conf' file should be located in this bot's (zulip_bots/bots/yoda/yoda)
            directory.
            Example input:
            @mention-bot You will learn how to speak like me someday.
            '''
        self.assert_bot_response(
            message = {'content': 'help'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # Test empty message.
        bot_response = '''
            This bot allows users to translate a sentence into
            'Yoda speak'.
            Users should preface messages with '@mention-bot'.

            Before running this, make sure to get a Mashape Api token.
            Instructions are in the 'readme.md' file.
            Store it in the 'yoda.conf' file.
            The 'yoda.conf' file should be located in this bot's (zulip_bots/bots/yoda/yoda)
            directory.
            Example input:
            @mention-bot You will learn how to speak like me someday.
            '''
        self.assert_bot_response(
            message = {'content': ''},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # Test invalid input.
        bot_response = "Invalid input, please check the sentence you have entered."
        with self.mock_config_info({'api_key': '12345678'}), \
                self.mock_http_conversation('test_invalid_input'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': '@#$%^&*'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestDefineBot(BotTestCase):
    bot_name = "define"

    def test_bot(self):

        # Only one type(noun) of word.
        bot_response = ("**cat**:\n\n* (**noun**) a small domesticated carnivorous mammal "
                        "with soft fur, a short snout, and retractile claws. It is widely "
                        "kept as a pet or for catching mice, and many breeds have been "
                        "developed.\n&nbsp;&nbsp;their pet cat\n\n")
        with self.mock_http_conversation('test_single_type_word'):
            self.assert_bot_response(
                message = {'content': 'cat'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Multi-type word.
        bot_response = ("**help**:\n\n"
                        "* (**verb**) make it easier or possible for (someone) to do something by offering them one's services or resources.\n"
                        "&nbsp;&nbsp;they helped her with domestic chores\n\n\n"
                        "* (**verb**) serve someone with (food or drink).\n"
                        "&nbsp;&nbsp;may I help you to some more meat?\n\n\n"
                        "* (**verb**) cannot or could not avoid.\n"
                        "&nbsp;&nbsp;he couldn't help laughing\n\n\n"
                        "* (**noun**) the action of helping someone to do something.\n"
                        "&nbsp;&nbsp;I asked for help from my neighbours\n\n\n"
                        "* (**exclamation**) used as an appeal for urgent assistance.\n"
                        "&nbsp;&nbsp;Help! I'm drowning!\n\n")
        with self.mock_http_conversation('test_multi_type_word'):
            self.assert_bot_response(
                message = {'content': 'help'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Incorrect word.
        bot_response = "**foo**:\nCould not load definition."
        with self.mock_http_conversation('test_incorrect_word'):
            self.assert_bot_response(
                message = {'content': 'foo'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Phrases are not defined. No request is sent to the Internet.
        bot_response = "Definitions for phrases are not available."
        self.assert_bot_response(
            message = {'content': 'The sky is blue'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # Symbols are considered invalid for words
        bot_response = "Definitions of words with symbols are not possible."
        self.assert_bot_response(
            message = {'content': '#'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # Empty messages are returned with a prompt to reply. No request is sent to the Internet.
        bot_response = "Please enter a word to define."
        self.assert_bot_response(
            message = {'content': ''},
            response = {'content': bot_response},
            expected_method='send_reply'
        )



# See readme.md for instructions on running this code.

from __future__ import print_function
import logging
import ssl
import sys
try:
    import requests
except ImportError as e:
    logging.error("Dependency missing!!\n{}".format(e))
    sys.exit(0)

HELP_MESSAGE = '''
            This bot allows users to translate a sentence into
            'Yoda speak'.
            Users should preface messages with '@mention-bot'.

            Before running this, make sure to get a Mashape Api token.
            Instructions are in the 'readme.md' file.
            Store it in the 'yoda.conf' file.
            The 'yoda.conf' file should be located in this bot's (zulip_bots/bots/yoda/yoda)
            directory.
            Example input:
            @mention-bot You will learn how to speak like me someday.
            '''


class ApiKeyError(Exception):
    '''raise this when there is an error with the Mashape Api Key'''

class ServiceUnavailableError(Exception):
    '''raise this when the service is unavailable.'''


class YodaSpeakHandler(object):
    '''
    This bot will allow users to translate a sentence into 'Yoda speak'.
    It looks for messages starting with '@mention-bot'.
    '''
    def initialize(self, bot_handler):
        self.api_key = bot_handler.get_config_info('yoda')['api_key']

    def usage(self):
        return '''
            This bot will allow users to translate a sentence into
            'Yoda speak'.
            Users should preface messages with '@mention-bot'.

            Before running this, make sure to get a Mashape Api token.
            Instructions are in the 'readme.md' file.
            Store it in the 'yoda.conf' file.
            The 'yoda.conf' file should be located in this bot's directory.
            Example input:
            @mention-bot You will learn how to speak like me someday.
            '''

    def handle_message(self, message, bot_handler):
        self.handle_input(message, bot_handler)

    def send_to_yoda_api(self, sentence):
        # function for sending sentence to api
        response = requests.get("https://yoda.p.mashape.com/yoda?sentence=" + sentence,
                                headers={
                                    "X-Mashape-Key": self.api_key,
                                    "Accept": "text/plain"
                                }
                                )

        if response.status_code == 200:
            return response.text
        if response.status_code == 403:
            raise ApiKeyError
        if response.status_code == 503:
            raise ServiceUnavailableError
        else:
            error_message = response.text['message']
            logging.error(error_message)
            error_code = response.status_code
            error_message = error_message + 'Error code: ' + error_code +\
                ' Did you follow the instructions in the `readme.md` file?'
            return error_message

    def format_input(self, original_content):
        # gets rid of whitespace around the edges, so that they aren't a problem in the future
        message_content = original_content.strip()
        # replaces all spaces with '+' to be in the format the api requires
        sentence = message_content.replace(' ', '+')
        return sentence

    def handle_input(self, message, bot_handler):

        original_content = message['content']

        if self.is_help(original_content) or (original_content == ""):
            bot_handler.send_reply(message, HELP_MESSAGE)

        else:
            sentence = self.format_input(original_content)
            try:
                reply_message = self.send_to_yoda_api(sentence)

                if len(reply_message) == 0:
                    reply_message = 'Invalid input, please check the sentence you have entered.'

            except ssl.SSLError or TypeError:
                reply_message = 'The service is temporarily unavailable, please try again.'
                logging.error(reply_message)

            except ApiKeyError:
                reply_message = 'Invalid Api Key. Did you follow the instructions in the ' \
                                '`readme.md` file?'
                logging.error(reply_message)

            bot_handler.send_reply(message, reply_message)

    def send_message(self, bot_handler, message, stream, subject):
        # function for sending a message
        bot_handler.send_message(dict(
            type='stream',
            to=stream,
            subject=subject,
            content=message
        ))

    def is_help(self, original_content):
        # gets rid of whitespace around the edges, so that they aren't a problem in the future
        message_content = original_content.strip()
        if message_content == 'help':
            return True
        else:
            return False

handler_class = YodaSpeakHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

import json

from zulip_bots.test_lib import BotTestCase

class TestGiphyBot(BotTestCase):
    bot_name = "giphy"

    def test_bot(self):
        bot_response = '[Click to enlarge]' \
                       '(https://media4.giphy.com/media/3o6ZtpxSZbQRRnwCKQ/giphy.gif)' \
                       '[](/static/images/interactive-bot/giphy/powered-by-giphy.png)'

        with self.mock_config_info({'key': '12345678'}), \
                self.mock_http_conversation('test_1'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'Hello'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestFollowUpBot(BotTestCase):
    bot_name = "followup"

    def test_bot(self):
        expected_send_reply = [
            ("", 'Please specify the message you want to send to followup stream after @mention-bot')
        ]
        self.check_expected_responses(expected_send_reply, expected_method='send_reply')

        expected_send_message = [
            ("foo",
             {'type': 'stream',
              'to': 'followup',
              'subject': 'foo_sender@zulip.com',
              'content': 'from foo_sender@zulip.com: foo'}),
            ("I have completed my task",
             {'type': 'stream',
              'to': 'followup',
              'subject': 'foo_sender@zulip.com',
              'content': 'from foo_sender@zulip.com: I have completed my task'}),
        ]
        self.check_expected_responses(expected_send_message, expected_method='send_message')



#!/usr/bin/env python
from __future__ import absolute_import

import importlib
import os
import zulip_bots.run
from zulip_bots.lib import extract_query_without_mention
import six
import unittest
import zulip

from importlib import import_module
from typing import Optional
from unittest import TestCase

if six.PY2:
    import mock
    from mock import patch
else:
    from unittest import mock
    from unittest.mock import patch

class TestDefaultArguments(TestCase):

    our_dir = os.path.dirname(__file__)
    path_to_bot = os.path.abspath(os.path.join(our_dir, 'bots/giphy/giphy.py'))

    @patch('sys.argv', ['zulip-run-bot', 'giphy', '--config-file', '/foo/bar/baz.conf'])
    @patch('zulip_bots.run.run_message_handler_for_bot')
    def test_argument_parsing_with_bot_name(self, mock_run_message_handler_for_bot):
        # type: (mock.Mock) -> None
        with patch('zulip_bots.run.exit_gracefully_if_config_file_does_not_exist'):
            zulip_bots.run.main()

        mock_run_message_handler_for_bot.assert_called_with(bot_name='giphy',
                                                            config_file='/foo/bar/baz.conf',
                                                            lib_module=mock.ANY,
                                                            quiet=False)

    @patch('sys.argv', ['zulip-run-bot', path_to_bot, '--config-file', '/foo/bar/baz.conf'])
    @patch('zulip_bots.run.run_message_handler_for_bot')
    def test_argument_parsing_with_bot_path(self, mock_run_message_handler_for_bot):
        # type: (mock.Mock) -> None
        with patch('zulip_bots.run.exit_gracefully_if_config_file_does_not_exist'):
            zulip_bots.run.main()

        mock_run_message_handler_for_bot.assert_called_with(
            bot_name='giphy',
            config_file='/foo/bar/baz.conf',
            lib_module=mock.ANY,
            quiet=False)

class TestBotLib(TestCase):
    def test_extract_query_without_mention(self):
        # type: () -> None

        def test_message(name, message, expected_return):
            # type: (str, str, Optional[str]) -> None
            mock_client = mock.MagicMock()
            mock_client.full_name = name
            mock_message = {'content': message}
            self.assertEqual(expected_return, extract_query_without_mention(mock_message, mock_client))
        test_message("xkcd", "@**xkcd**foo", "foo")
        test_message("xkcd", "@**xkcd** foo", "foo")
        test_message("xkcd", "@**xkcd** foo bar baz", "foo bar baz")
        test_message("xkcd", "@**xkcd**         foo bar baz", "foo bar baz")
        test_message("xkcd", "@**xkcd** 123_) (/&%) +}}}l", "123_) (/&%) +}}}l")
        test_message("brokenmention", "@**brokenmention* foo", None)
        test_message("nomention", "foo", None)
        test_message("Max Mustermann", "@**Max Mustermann** foo", "foo")
        test_message("Max (Mustermann)#(*$&12]\]", "@**Max (Mustermann)#(*$&12]\]** foo", "foo")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase
from zulip_bots.lib import StateHandler

class TestTictactoeBot(BotTestCase):
    bot_name = "tictactoe"

    def test_bot(self):
        messages = [  # Template for message inputs to test, absent of message content
            {
                'type': 'stream',
                'display_recipient': 'some stream',
                'subject': 'some subject',
                'sender_email': 'foo_sender@zulip.com',
            },
            {
                'type': 'private',
                'sender_email': 'foo_sender@zulip.com',
            },
        ]
        private_response = {
            'type': 'private',
            'to': 'foo_sender@zulip.com',
            'subject': 'foo_sender@zulip.com',  # FIXME Requiring this in bot is a bug?
        }

        msg = dict(
            help = "*Help for Tic-Tac-Toe bot* \nThe bot responds to messages starting with @mention-bot.\n**@mention-bot new** will start a new game (but not if you're already in the middle of a game). You must type this first to start playing!\n**@mention-bot help** will return this help function.\n**@mention-bot quit** will quit from the current game.\n**@mention-bot <coordinate>** will make a move at the given coordinate.\nCoordinates are entered in a (row, column) format. Numbering is from top to bottom and left to right. \nHere are the coordinates of each position. (Parentheses and spaces are optional). \n(1, 1)  (1, 2)  (1, 3) \n(2, 1)  (2, 2)  (2, 3) \n(3, 1) (3, 2) (3, 3) \n",
            didnt_understand = "Hmm, I didn't understand your input. Type **@tictactoe help** or **@ttt help** to see valid inputs.",
            new_game = "Welcome to tic-tac-toe! You'll be x's and I'll be o's. Your move first!\nCoordinates are entered in a (row, column) format. Numbering is from top to bottom and left to right.\nHere are the coordinates of each position. (Parentheses and spaces are optional.) \n(1, 1)  (1, 2)  (1, 3) \n(2, 1)  (2, 2)  (2, 3) \n(3, 1) (3, 2) (3, 3) \n Your move would be one of these. To make a move, type @mention-bot followed by a space and the coordinate.",
            already_playing = "You're already playing a game! Type **@tictactoe help** or **@ttt help** to see valid inputs.",
            already_played_there = 'That space is already filled, sorry!',
            successful_quit = "You've successfully quit the game.",
            after_1_1 = ("[ x _ _ ]\n[ _ _ _ ]\n[ _ _ _ ]\n"
                         "My turn:\n[ x _ _ ]\n[ _ o _ ]\n[ _ _ _ ]\n"
                         "Your turn! Enter a coordinate or type help."),
            after_2_1 = ("[ x _ _ ]\n[ x o _ ]\n[ _ _ _ ]\n"
                         "My turn:\n[ x _ _ ]\n[ x o _ ]\n[ o _ _ ]\n"
                         "Your turn! Enter a coordinate or type help."),
            after_1_3 = ("[ x _ x ]\n[ x o _ ]\n[ o _ _ ]\n"
                         "My turn:\n[ x o x ]\n[ x o _ ]\n[ o _ _ ]\n"
                         "Your turn! Enter a coordinate or type help."),
            after_3_2 = ("[ x o x ]\n[ x o _ ]\n[ o x _ ]\n"
                         "My turn:\n[ x o x ]\n[ x o o ]\n[ o x _ ]\n"
                         "Your turn! Enter a coordinate or type help."),
        )

        expected_send_message = [
            # Empty message
            ("", msg['didnt_understand']),
            # Non-command
            ("adboh", msg['didnt_understand']),
            # Help command
            ("help", msg['help']),
            # Command: quit not understood with no game
            ("quit", msg['didnt_understand']),
            # Can quit if new game and have state
            ("new", msg['new_game']),
            ("quit", msg['successful_quit']),
            # Quit not understood when no game FIXME improve response?
            ("quit", msg['didnt_understand']),
            # New right after new just restarts
            ("new", msg['new_game']),
            ("new", msg['new_game']),
            # Make a corner play
            ("(1,1)", msg['after_1_1']),
            # New while playing doesn't just restart
            ("new", msg['already_playing']),
            # User played in this location already
            ("(1,1)", msg['already_played_there']),
            # ... and bot played here
            ("(2,2)", msg['already_played_there']),
            ("quit", msg['successful_quit']),
            # Can't play without game FIXME improve response?
            ("(1,1)", msg['didnt_understand']),
            ("new", msg['new_game']),
            # Value out of range FIXME improve response?
            ("(1,5)", msg['didnt_understand']),
            # Value out of range FIXME improve response?
            ("0,1", msg['didnt_understand']),
            # Sequence of moves to show valid input formats:
            ("1,1", msg['after_1_1']),
            ("2, 1", msg['after_2_1']),
            ("(1,3)", msg['after_1_3']),
            ("quit", msg['successful_quit']),
            # Can't test 'after_3_2' as it's random!
        ]
        for m in messages:
            for (mesg, resp) in expected_send_message:
                self.assert_bot_response(dict(m, content=mesg),
                                         dict(private_response, content=resp),
                                         'send_message')



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestEncryptBot(BotTestCase):
    bot_name = "encrypt"

    def test_bot(self):
        expected = [
            ("", "Encrypted/Decrypted text: "),
            ("Let\'s Do It", "Encrypted/Decrypted text: Yrg\'f Qb Vg"),
            ("me&mom together..!!", "Encrypted/Decrypted text: zr&zbz gbtrgure..!!"),
            ("foo bar", "Encrypted/Decrypted text: sbb one"),
            ("Please encrypt this", "Encrypted/Decrypted text: Cyrnfr rapelcg guvf"),
        ]
        self.check_expected_responses(expected)



from __future__ import absolute_import
from __future__ import print_function
import requests
import logging
import re
from six.moves import urllib

# See readme.md for instructions on running this code.

class WikipediaHandler(object):
    '''
    This plugin facilitates searching Wikipedia for a
    specific key term and returns the top article from the
    search. It looks for messages starting with '@mention-bot'

    In this example, we write all Wikipedia searches into
    the same stream that it was called from, but this code
    could be adapted to write Wikipedia searches to some
    kind of external issue tracker as well.
    '''

    META = {
        'name': 'Wikipedia',
        'description': 'Searches Wikipedia for a term and returns the top article.',
    }

    def usage(self):
        return '''
            This plugin will allow users to directly search
            Wikipedia for a specific key term and get the top
            article that is returned from the search. Users
            should preface searches with "@mention-bot".
            '''

    def handle_message(self, message, bot_handler):
        bot_response = self.get_bot_wiki_response(message, bot_handler)
        bot_handler.send_reply(message, bot_response)

    def get_bot_wiki_response(self, message, bot_handler):
        help_text = 'Please enter your message after @mention-bot'
        query = message['content']
        if query == '':
            return help_text
        query_wiki_link = ('https://en.wikipedia.org/w/api.php?action=query&'
                           'list=search&srsearch=%s&format=json'
                           % (urllib.parse.quote(query),))
        try:
            data = requests.get(query_wiki_link)
        except requests.exceptions.RequestException:
            logging.error('broken link')
            return

        if data.status_code != 200:
            logging.error('unsuccessful data')
            return

        new_content = 'For search term "' + query
        if len(data.json()['query']['search']) == 0:
            new_content = 'I am sorry. The search term you provided is not found :slightly_frowning_face:'
        else:
            search_string = data.json()['query']['search'][0]['title'].replace(' ', '_')
            url = 'https://en.wikipedia.org/wiki/' + search_string
            new_content = new_content + '", ' + url
        return new_content


handler_class = WikipediaHandler



# See readme.md for instructions on running this code.

class HelpHandler(object):
    def usage(self):
        return '''
            This plugin will give info about Zulip to
            any user that types a message saying "help".

            This is example code; ideally, you would flesh
            this out for more useful help pertaining to
            your Zulip instance.
            '''

    def handle_message(self, message, bot_handler):
        help_content = "Info on Zulip can be found here:\nhttps://github.com/zulip/zulip"
        bot_handler.send_reply(message, help_content)

handler_class = HelpHandler



# A dictionary allowing the conversion of each unit to its base unit.
# An entry consists of the unit's name, a constant number and a constant
# factor that need to be added and multiplied to convert the unit into
# the base unit in the last parameter.
UNITS = {'bit': [0, 1, 'bit'],
         'byte': [0, 8, 'bit'],
         'cubic-centimeter': [0, 0.000001, 'cubic-meter'],
         'cubic-decimeter': [0, 0.001, 'cubic-meter'],
         'liter': [0, 0.001, 'cubic-meter'],
         'cubic-meter': [0, 1, 'cubic-meter'],
         'cubic-inch': [0, 0.000016387064, 'cubic-meter'],
         'fluid-ounce': [0, 0.000029574, 'cubic-meter'],
         'cubic-foot': [0, 0.028316846592, 'cubic-meter'],
         'cubic-yard': [0, 0.764554857984, 'cubic-meter'],
         'teaspoon': [0, 0.0000049289216, 'cubic-meter'],
         'tablespoon': [0, 0.000014787, 'cubic-meter'],
         'cup': [0, 0.00023658823648491, 'cubic-meter'],
         'gram': [0, 1, 'gram'],
         'kilogram': [0, 1000, 'gram'],
         'ton': [0, 1000000, 'gram'],
         'ounce': [0, 28.349523125, 'gram'],
         'pound': [0, 453.59237, 'gram'],
         'kelvin': [0, 1, 'kelvin'],
         'celsius': [273.15, 1, 'kelvin'],
         'fahrenheit': [255.372222, 0.555555, 'kelvin'],
         'centimeter': [0, 0.01, 'meter'],
         'decimeter': [0, 0.1, 'meter'],
         'meter': [0, 1, 'meter'],
         'kilometer': [0, 1000, 'meter'],
         'inch': [0, 0.0254, 'meter'],
         'foot': [0, 0.3048, 'meter'],
         'yard': [0, 0.9144, 'meter'],
         'mile': [0, 1609.344, 'meter'],
         'nautical-mile': [0, 1852, 'meter'],
         'square-centimeter': [0, 0.0001, 'square-meter'],
         'square-decimeter': [0, 0.01, 'square-meter'],
         'square-meter': [0, 1, 'square-meter'],
         'square-kilometer': [0, 1000000, 'square-meter'],
         'square-inch': [0, 0.00064516, 'square-meter'],
         'square-foot': [0, 0.09290304, 'square-meter'],
         'square-yard': [0, 0.83612736, 'square-meter'],
         'square-mile': [0, 2589988.110336, 'square-meter'],
         'are': [0, 100, 'square-meter'],
         'hectare': [0, 10000, 'square-meter'],
         'acre': [0, 4046.8564224, 'square-meter']}

PREFIXES = {'atto': -18,
            'femto': -15,
            'pico': -12,
            'nano': -9,
            'micro': -6,
            'milli': -3,
            'centi': -2,
            'deci': -1,
            'deca': 1,
            'hecto': 2,
            'kilo': 3,
            'mega': 6,
            'giga': 9,
            'tera': 12,
            'peta': 15,
            'exa': 18}

ALIASES = {'a': 'are',
           'ac': 'acre',
           'c': 'celsius',
           'cm': 'centimeter',
           'cm2': 'square-centimeter',
           'cm3': 'cubic-centimeter',
           'cm^2': 'square-centimeter',
           'cm^3': 'cubic-centimeter',
           'dm': 'decimeter',
           'dm2': 'square-decimeter',
           'dm3': 'cubic-decimeter',
           'dm^2': 'square-decimeter',
           'dm^3': 'cubic-decimeter',
           'f': 'fahrenheit',
           'fl-oz': 'fluid-ounce',
           'ft': 'foot',
           'ft2': 'square-foot',
           'ft3': 'cubic-foot',
           'ft^2': 'square-foot',
           'ft^3': 'cubic-foot',
           'g': 'gram',
           'ha': 'hectare',
           'in': 'inch',
           'in2': 'square-inch',
           'in3': 'cubic-inch',
           'in^2': 'square-inch',
           'in^3': 'cubic-inch',
           'k': 'kelvin',
           'kg': 'kilogram',
           'km': 'kilometer',
           'km2': 'square-kilometer',
           'km^2': 'square-kilometer',
           'l': 'liter',
           'lb': 'pound',
           'm': 'meter',
           'm2': 'square-meter',
           'm3': 'cubic-meter',
           'm^2': 'square-meter',
           'm^3': 'cubic-meter',
           'mi': 'mile',
           'mi2': 'square-mile',
           'mi^2': 'square-mile',
           'nmi': 'nautical-mile',
           'oz': 'ounce',
           't': 'ton',
           'tbsp': 'tablespoon',
           'tsp': 'teaspoon',
           'y': 'yard',
           'y2': 'square-yard',
           'y3': 'cubic-yard',
           'y^2': 'square-yard',
           'y^3': 'cubic-yard'}

HELP_MESSAGE = ('Converter usage:\n'
                '`@convert <number> <unit_from> <unit_to>`\n'
                'Converts `number` in the unit <unit_from> to '
                'the <unit_to> and prints the result\n'
                '`number`: integer or floating point number, e.g. 12, 13.05, 0.002\n'
                '<unit_from> and <unit_to> are two of the following units:\n'
                '* square-centimeter (cm^2, cm2), square-decimeter (dm^2, dm2), '
                'square-meter (m^2, m2), square-kilometer (km^2, km2),'
                ' square-inch (in^2, in2), square-foot (ft^2, ft2), square-yard (y^2, y2), '
                ' square-mile(mi^2, mi2),  are (a), hectare (ha), acre (ac)\n'
                '* bit, byte\n'
                '* centimeter (cm), decimeter(dm), meter (m),'
                ' kilometer (km), inch (in), foot (ft), yard (y),'
                ' mile (mi), nautical-mile (nmi)\n'
                '* Kelvin (K), Celsius(C), Fahrenheit (F)\n'
                '* cubic-centimeter (cm^3, cm3), cubic-decimeter (dm^3, dm3), liter (l), '
                'cubic-meter (m^3, m3), cubic-inch (in^3, in3), fluid-ounce (fl-oz), '
                'cubic-foot (ft^3, ft3), cubic-yard (y^3, y3)\n'
                '* gram (g), kilogram (kg), ton (t), ounce (oz), pound(lb)\n'
                '* (metric only, U.S. and imperial units differ slightly:) teaspoon (tsp), tablespoon (tbsp), cup\n\n\n'
                'Allowed prefixes are:\n'
                '* atto, pico, femto, nano, micro, milli, centi, deci\n'
                '* deca, hecto, kilo, mega, giga, tera, peta, exa\n\n\n'
                'Usage examples:\n'
                '* `@convert 12 celsius fahrenheit`\n'
                '* `@convert 0.002 kilomile millimeter`\n'
                '* `@convert 31.5 square-mile ha`\n'
                '* `@convert 56 g lb`\n')

QUICK_HELP = 'Enter `@convert help` for help on using the converter.'



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestWikipediaBot(BotTestCase):
    bot_name = "wikipedia"

    def test_bot(self):

        # Single-word query
        bot_response = "For search term \"happy\", https://en.wikipedia.org/wiki/Happiness"
        with self.mock_http_conversation('test_single_word'):
            self.assert_bot_response(
                message = {'content': 'happy'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Multi-word query
        bot_response = "For search term \"The sky is blue\", https://en.wikipedia.org/wiki/Sky_blue"
        with self.mock_http_conversation('test_multi_word'):
            self.assert_bot_response(
                message = {'content': 'The sky is blue'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Number query
        bot_response = "For search term \"123\", https://en.wikipedia.org/wiki/123"
        with self.mock_http_conversation('test_number_query'):
            self.assert_bot_response(
                message = {'content': '123'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Hash query
        bot_response = "For search term \"#\", https://en.wikipedia.org/wiki/Number_sign"
        with self.mock_http_conversation('test_hash_query'):
            self.assert_bot_response(
                message = {'content': '#'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Incorrect word
        bot_response = "I am sorry. The search term you provided is not found :slightly_frowning_face:"
        with self.mock_http_conversation('test_incorrect_query'):
            self.assert_bot_response(
                message = {'content': 'sssssss kkkkk'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Empty query, no request made to the Internet.
        bot_response = "Please enter your message after @mention-bot"
        self.assert_bot_response(
            message = {'content': ''},
            response = {'content': bot_response},
            expected_method='send_reply'
        )



# See readme.md for instructions on running this code.
from __future__ import print_function
import requests
import json

class WeatherHandler(object):
    def initialize(self, bot_handler):
        self.api_key = bot_handler.get_config_info('weather')['key']
        self.response_pattern = 'Weather in {}, {}:\n{:.2f} F / {:.2f} C\n{}'

    def usage(self):
        return '''
            This plugin will give info about weather in a specified city
            '''

    def handle_message(self, message, bot_handler):
        help_content = '''
            This bot returns weather info for specified city.
            You specify city in the following format:
            city, state/country
            state and country parameter is optional(useful when there are many cities with the same name)
            For example:
            @**Weather Bot** Portland
            @**Weather Bot** Portland, Me
            '''.strip()

        if (message['content'] == 'help') or (message['content'] == ''):
            response = help_content
        else:
            url = 'http://api.openweathermap.org/data/2.5/weather?q=' + message['content'] + '&APPID='
            r = requests.get(url + self.api_key)
            if r.json()['cod'] == "404":
                response = "Sorry, city not found"
            else:
                response = format_response(r, message['content'], self.response_pattern)

        bot_handler.send_reply(message, response)


def format_response(text, city, response_pattern):
    j = text.json()
    city = j['name']
    country = j['sys']['country']
    fahrenheit = to_fahrenheit(j['main']['temp'])
    celsius = to_celsius(j['main']['temp'])
    description = j['weather'][0]['description'].title()

    return response_pattern.format(city, country, fahrenheit, celsius, description)


def to_celsius(temp_kelvin):
    return int(temp_kelvin) - 273.15


def to_fahrenheit(temp_kelvin):
    return int(temp_kelvin) * (9. / 5.) - 459.67

handler_class = WeatherHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from six.moves import zip

from zulip_bots.test_lib import BotTestCase

class TestHelloWorldBot(BotTestCase):
    bot_name = "helloworld"

    def test_bot(self):
        txt = "beep boop"
        messages = ["", "foo", "Hi, my name is abc"]
        self.check_expected_responses(list(zip(messages, len(messages)*[txt])))



#!/usr/bin/env python
from __future__ import print_function
from __future__ import absolute_import

import os
import sys
import argparse
import zulip_bots

from six.moves import configparser
from typing import Any

from mock import MagicMock, patch
from zulip_bots.lib import StateHandler
from zulip_bots.lib import ExternalBotHandler
from zulip_bots.provision import provision_bot
from zulip_bots.run import import_module_from_source

current_dir = os.path.dirname(os.path.abspath(__file__))

def parse_args():
    # type: () -> argparse.Namespace
    description = (
        "A tool to test a bot: given a provided message, provides the bot response.\n\n"
        'Examples:   %(prog)s xkcd 1\n'
        '            %(prog)s ./bot_folder/my_own_bot.py "test message"')
    epilog = (
        "The message need only be enclosed in quotes if empty or containing spaces.\n\n"
        "(Internally, this program loads bot-related code from the library code and\n"
        "then feeds the message provided in the command to the library code to handle.)")

    parser = argparse.ArgumentParser(description=description,
                                     epilog=epilog,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('bot',
                        action='store',
                        help='the name or path an existing bot to run')

    parser.add_argument('message',
                        action='store',
                        help='the message content to send to the bot')

    parser.add_argument('--force', '-f',
                        action='store_true',
                        help='try running bot even if dependencies install fails')

    parser.add_argument('--provision', '-p',
                        action='store_true',
                        help='install dependencies for the bot')

    args = parser.parse_args()
    return args

def main():
    # type: () -> None
    args = parse_args()
    if os.path.isfile(args.bot):
        bot_path = os.path.abspath(args.bot)
        bot_name = os.path.splitext(os.path.basename(bot_path))[0]
    else:
        bot_path = os.path.abspath(os.path.join(current_dir, 'bots', args.bot, args.bot+'.py'))
        bot_name = args.bot
    bot_dir = os.path.dirname(bot_path)
    if args.provision:
        provision_bot(os.path.dirname(bot_path), args.force)
    try:
        lib_module = import_module_from_source(bot_path, bot_name)
    except IOError:
        print("Could not find and import bot '{}'".format(bot_name))
        sys.exit(1)

    message = {'content': args.message, 'sender_email': 'foo_sender@zulip.com'}
    try:
        message_handler = lib_module.handler_class()
    except AttributeError:
        print("This module does not appear to have a bot handler_class specified.")
        sys.exit(1)

    with patch('zulip.Client') as mock_client:
        mock_bot_handler = ExternalBotHandler(mock_client, bot_dir)  # type: Any
        mock_bot_handler.send_reply = MagicMock()
        mock_bot_handler.send_message = MagicMock()
        mock_bot_handler.update_message = MagicMock()
        if hasattr(message_handler, 'initialize') and callable(message_handler.initialize):
            message_handler.initialize(mock_bot_handler)
        message_handler.handle_message(
            message=message,
            bot_handler=mock_bot_handler
        )
        print("On sending {} bot the message \"{}\"".format(bot_name, args.message))
        # send_reply and send_message have slightly arguments; the
        # following takes that into account.
        #   send_reply(original_message, response)
        #   send_message(response_message)
        if mock_bot_handler.send_reply.called:
            output_message = list(mock_bot_handler.send_reply.call_args)[0][1]
        elif mock_bot_handler.send_message.called:
            output_message = list(mock_bot_handler.send_message.call_args)[0][0]
        elif mock_bot_handler.update_message.called:
            output_message = list(mock_bot_handler.update_message.call_args)[0][0]['content']
            print("the bot updates a message with the following text (in quotes):\n\"{}\"".format(output_message))
            sys.exit()
        else:
            print("the bot sent no reply.")
            sys.exit()
        print("the bot gives the following output message (in quotes):\n\"{}\"".format(output_message))

if __name__ == '__main__':
    main()



# See readme.md for instructions on running this code.

import re
import os

class VirtualFsHandler(object):
    META = {
        'name': 'VirtualFs',
        'description': 'Provides a simple, permanent file system to store and retrieve strings.',
    }

    def usage(self):
        return get_help()

    def handle_message(self, message, bot_handler):
        command = message['content']
        if command == "":
            command = "help"

        sender = message['sender_email']

        recipient = message['display_recipient']
        if isinstance(recipient, list):  # If not a stream, then hash on list of emails
            recipient = " ".join([x['email'] for x in recipient])

        storage = bot_handler.storage
        if not storage.contains(recipient):
            storage.put(recipient, fs_new())
        fs = storage.get(recipient)
        if sender not in fs['user_paths']:
            fs['user_paths'][sender] = '/'
        fs, msg = fs_command(fs, sender, command)
        prependix = '{}:\n'.format(sender)
        msg = prependix + msg
        storage.put(recipient, fs)

        bot_handler.send_reply(message, msg)


def get_help():
    return '''
This bot implements a virtual file system for a stream.
The locations of text are persisted for the lifetime of the bot
running, and if you rename a stream, you will lose the info.
Example commands:

```
@mention-bot sample_conversation: sample conversation with the bot
@mention-bot mkdir: create a directory
@mention-bot ls: list a directory
@mention-bot cd: change directory
@mention-bot pwd: show current path
@mention-bot write: write text
@mention-bot read: read text
@mention-bot rm: remove a file
@mention-bot rmdir: remove a directory
```
Use commands like `@mention-bot help write` for more details on specific
commands.
'''

def sample_conversation():
    return [
        ('cd /\nCurrent path: /\n\n'),
        ('cd /home\nERROR: invalid path\n\n'),
        ('cd .\nERROR: invalid path\n\n'),
        ('mkdir home\ndirectory created\n\n'),
        ('cd home\nCurrent path: /home/\n\n'),
        ('cd /home/\nCurrent path: /home/\n\n'),
        ('mkdir stuff/\nERROR: stuff/ is not a valid name\n\n'),
        ('mkdir stuff\ndirectory created\n\n'),
        ('write stuff/file1 something\nfile written\n\n'),
        ('read stuff/file1\nsomething\n\n'),
        ('read /home/stuff/file1\nsomething\n\n'),
        ('read home/stuff/file1\nERROR: file does not exist\n\n'),
        ('pwd    \n/home/\n\n'),
        ('pwd bla\nERROR: syntax: pwd\n\n'),
        ('ls bla foo\nERROR: syntax: ls <optional_path>\n\n'),
        ('cd /\nCurrent path: /\n\n'),
        ('rm home\nERROR: /home/ is a directory, file required\n\n'),
        ('rmdir home\nremoved\n\n'),
        ('ls  \nWARNING: directory is empty\n\n'),
        ('cd home\nERROR: invalid path\n\n'),
        ('read /home/stuff/file1\nERROR: file does not exist\n\n'),
        ('cd /\nCurrent path: /\n\n'),
        ('write /foo contents of /foo\nfile written\n\n'),
        ('read /foo\ncontents of /foo\n\n'),
        ('write /bar Contents: bar bar\nfile written\n\n'),
        ('read /bar\nContents: bar bar\n\n'),
        ('write /bar invalid\nERROR: file already exists\n\n'),
        ('rm /bar\nremoved\n\n'),
        ('rm /bar\nERROR: file does not exist\n\n'),
        ('write /bar new bar\nfile written\n\n'),
        ('read /bar\nnew bar\n\n'),
        ('write /yo/invalid whatever\nERROR: /yo is not a directory\n\n'),
        ('mkdir /yo\ndirectory created\n\n'),
        ('read /yo\nERROR: /yo/ is a directory, file required\n\n'),
        ('ls /yo\nWARNING: directory is empty\n\n'),
        ('read /yo/nada\nERROR: file does not exist\n\n'),
        ('write /yo whatever\nERROR: file already exists\n\n'),
        ('write /yo/apple red\nfile written\n\n'),
        ('read /yo/apple\nred\n\n'),
        ('mkdir /yo/apple\nERROR: file already exists\n\n'),
        ('ls /invalid\nERROR: file does not exist\n\n'),
        ('ls /foo\nERROR: /foo is not a directory\n\n'),
        ('ls /\n* /*bar*\n* /*foo*\n* /yo/\n\n'),
        ('invalid command\nERROR: unrecognized command\n\n'),
        ('write\nERROR: syntax: write <path> <some_text>\n\n'),
        ('help' + get_help() + '\n\n'),
        ('help ls\nsyntax: ls <optional_path>\n\n'),
        ('help invalid_command' + get_help() + '\n\n'),
    ]

REGEXES = dict(
    command='(cd|ls|mkdir|read|rmdir|rm|write|pwd)',
    path='(\S+)',
    optional_path='(\S*)',
    some_text='(.+)',
)

def get_commands():
    return {
        'help': (fs_help, ['command']),
        'sample_conversation': (fs_sample_conversation, ['command']),
        'ls': (fs_ls, ['optional_path']),
        'mkdir': (fs_mkdir, ['path']),
        'read': (fs_read, ['path']),
        'rm': (fs_rm, ['path']),
        'rmdir': (fs_rmdir, ['path']),
        'write': (fs_write, ['path', 'some_text']),
        'cd': (fs_cd, ['path']),
        'pwd': (fs_pwd, []),
    }

def fs_command(fs, user, cmd):
    cmd = cmd.strip()
    if cmd == 'help':
        return fs, get_help()
    if cmd == 'sample_conversation':
        return fs, (''.join(sample_conversation()))
    cmd_name = cmd.split()[0]
    cmd_args = cmd[len(cmd_name):].strip()
    commands = get_commands()
    if cmd_name not in commands:
        return fs, 'ERROR: unrecognized command'

    f, arg_names = commands[cmd_name]
    partial_regexes = [REGEXES[a] for a in arg_names]
    regex = ' '.join(partial_regexes)
    regex += '$'
    m = re.match(regex, cmd_args)
    if m:
        return f(fs, user, *m.groups())
    elif cmd_name == 'help':
        return fs, get_help()
    else:
        return fs, 'ERROR: ' + syntax_help(cmd_name)

def syntax_help(cmd_name):
    commands = get_commands()
    f, arg_names = commands[cmd_name]
    arg_syntax = ' '.join('<' + a + '>' for a in arg_names)
    if arg_syntax:
        cmd = cmd_name + ' ' + arg_syntax
    else:
        cmd = cmd_name
    return 'syntax: {}'.format(cmd)

def fs_new():
    fs = {
        '/': directory([]),
        'user_paths': dict()
    }
    return fs

def fs_help(fs, user, cmd_name):
    return fs, syntax_help(cmd_name)

def fs_sample_conversation(fs, user, cmd_name):
    return fs, syntax_help(cmd_name)

def fs_mkdir(fs, user, fn):
    path, msg = make_path(fs, user, fn)
    if msg:
        return fs, msg
    if path in fs:
        return fs, 'ERROR: file already exists'
    dir_path = os.path.dirname(path)
    if not is_directory(fs, dir_path):
        msg = 'ERROR: {} is not a directory'.format(dir_path)
        return fs, msg
    new_fs = fs.copy()
    new_dir = directory({path}.union(fs[dir_path]['fns']))
    new_fs[dir_path] = new_dir
    new_fs[path] = directory([])
    msg = 'directory created'
    return new_fs, msg

def fs_ls(fs, user, fn):
    if fn == '.' or fn == '':
        path = fs['user_paths'][user]
    else:
        path, msg = make_path(fs, user, fn)
        if msg:
            return fs, msg
    if path not in fs:
        msg = 'ERROR: file does not exist'
        return fs, msg
    if not is_directory(fs, path):
        return fs, 'ERROR: {} is not a directory'.format(path)
    fns = fs[path]['fns']
    if not fns:
        return fs, 'WARNING: directory is empty'
    msg = '\n'.join('* ' + nice_path(fs, path) for path in sorted(fns))
    return fs, msg

def fs_pwd(fs, user):
    path = fs['user_paths'][user]
    msg = nice_path(fs, path)
    return fs, msg

def fs_rm(fs, user, fn):
    path, msg = make_path(fs, user, fn)
    if msg:
        return fs, msg
    if path not in fs:
        msg = 'ERROR: file does not exist'
        return fs, msg
    if fs[path]['kind'] == 'dir':
        msg = 'ERROR: {} is a directory, file required'.format(nice_path(fs, path))
        return fs, msg
    new_fs = fs.copy()
    new_fs.pop(path)
    directory = get_directory(path)
    new_fs[directory]['fns'].remove(path)
    msg = 'removed'
    return new_fs, msg

def fs_rmdir(fs, user, fn):
    path, msg = make_path(fs, user, fn)
    if msg:
        return fs, msg
    if path not in fs:
        msg = 'ERROR: directory does not exist'
        return fs, msg
    if fs[path]['kind'] == 'text':
        msg = 'ERROR: {} is a file, directory required'.format(nice_path(fs, path))
        return fs, msg
    new_fs = fs.copy()
    new_fs.pop(path)
    directory = get_directory(path)
    new_fs[directory]['fns'].remove(path)
    for sub_path in new_fs.keys():
        if sub_path.startswith(path+'/'):
            new_fs.pop(sub_path)
    msg = 'removed'
    return new_fs, msg

def fs_write(fs, user, fn, content):
    path, msg = make_path(fs, user, fn)
    if msg:
        return fs, msg
    if path in fs:
        msg = 'ERROR: file already exists'
        return fs, msg
    dir_path = os.path.dirname(path)
    if not is_directory(fs, dir_path):
        msg = 'ERROR: {} is not a directory'.format(dir_path)
        return fs, msg
    new_fs = fs.copy()
    new_dir = directory({path}.union(fs[dir_path]['fns']))
    new_fs[dir_path] = new_dir
    new_fs[path] = text_file(content)
    msg = 'file written'
    return new_fs, msg

def fs_read(fs, user, fn):
    path, msg = make_path(fs, user, fn)
    if msg:
        return fs, msg
    if path not in fs:
        msg = 'ERROR: file does not exist'
        return fs, msg
    if fs[path]['kind'] == 'dir':
        msg = 'ERROR: {} is a directory, file required'.format(nice_path(fs, path))
        return fs, msg
    val = fs[path]['content']
    return fs, val

def fs_cd(fs, user, fn):
    if len(fn) > 1 and fn[-1] == '/':
        fn = fn[:-1]
    path = fn if len(fn) > 0 and fn[0] == '/' else make_path(fs, user, fn)[0]
    if path not in fs:
        msg = 'ERROR: invalid path'
        return fs, msg
    if fs[path]['kind'] == 'text':
        msg = 'ERROR: {} is a file, directory required'.format(nice_path(fs, path))
        return fs, msg
    fs['user_paths'][user] = path
    return fs, "Current path: {}".format(nice_path(fs, path))

def make_path(fs, user, leaf):
    if leaf == '/':
        return ['/', '']
    if leaf.endswith('/'):
        return ['', 'ERROR: {} is not a valid name'.format(leaf)]
    if leaf.startswith('/'):
        return [leaf, '']
    path = fs['user_paths'][user]
    if not path.endswith('/'):
        path += '/'
    path += leaf
    return path, ''

def nice_path(fs, path):
    path_nice = path
    slash = path.rfind('/')
    if path not in fs:
        return 'ERROR: the current directory does not exist'
    if fs[path]['kind'] == 'text':
        path_nice = '{}*{}*'.format(path[:slash+1], path[slash+1:])
    elif path != '/':
        path_nice = '{}/'.format(path)
    return path_nice

def get_directory(path):
    slash = path.rfind('/')
    if slash == 0:
        return '/'
    else:
        return path[:slash]

def directory(fns):
    return dict(kind='dir', fns=list(fns))

def text_file(content):
    return dict(kind='text', content=content)

def is_directory(fs, fn):
    if fn not in fs:
        return False
    return fs[fn]['kind'] == 'dir'

handler_class = VirtualFsHandler



#!/usr/bin/env python
from __future__ import print_function
from __future__ import absolute_import

import logging
import argparse
import sys
import os
from types import ModuleType
from importlib import import_module
from os.path import basename, splitext
from typing import Any, Optional, Text

from zulip_bots.lib import run_message_handler_for_bot
from zulip_bots.provision import provision_bot

current_dir = os.path.dirname(os.path.abspath(__file__))

def import_module_from_source(path, name=None):
    # type: (Text, Optional[Text]) -> Any
    if not name:
        name = splitext(basename(path))[0]

    # importlib.util.module_from_spec is supported from Python3.5
    py_version = sys.version_info
    if py_version.major < 3 or (py_version.major == 3 and py_version.minor < 5):
        import imp
        module = imp.load_source(name, path)
    else:
        import importlib.util
        spec = importlib.util.spec_from_file_location(name, path)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

    return module

def name_and_path_match(given_name, path_to_bot):
    # type: (Text, Text) -> bool
    if given_name and path_to_bot:
        name_by_path = os.path.splitext(os.path.basename(path_to_bot))[0]
        if (given_name != name_by_path):
            return False
    return True

def parse_args():
    # type: () -> argparse.Namespace
    usage = '''
        zulip-run-bot <bot_name> --config-file ~/zuliprc
        zulip-run-bot --help
        '''

    parser = argparse.ArgumentParser(usage=usage)
    parser.add_argument('bot',
                        action='store',
                        help='the name or path of an existing bot to run')

    parser.add_argument('--quiet', '-q',
                        action='store_true',
                        help='turn off logging output')

    parser.add_argument('--config-file', '-c',
                        action='store',
                        required=True,
                        help='zulip configuration file (e.g. ~/Downloads/zuliprc)')

    parser.add_argument('--force',
                        action='store_true',
                        help='try running the bot even if dependencies install fails')

    parser.add_argument('--provision',
                        action='store_true',
                        help='install dependencies for the bot')

    args = parser.parse_args()
    return args


def exit_gracefully_if_config_file_does_not_exist(config_file):
    # type: (str) -> None
    if not os.path.exists(config_file):
        print('''
            ERROR: %s does not exist.

            You may need to download a config file from the Zulip app, or
            if you have already done that, you need to specify the file
            location correctly.
            ''' % (config_file,))
        sys.exit(1)

def main():
    # type: () -> None
    args = parse_args()
    if os.path.isfile(args.bot):
        bot_path = os.path.abspath(args.bot)
        bot_name = os.path.splitext(basename(bot_path))[0]
    else:
        bot_path = os.path.abspath(os.path.join(current_dir, 'bots', args.bot, args.bot+'.py'))
        bot_name = args.bot
    if args.provision:
        provision_bot(os.path.dirname(bot_path), args.force)
    lib_module = import_module_from_source(bot_path, bot_name)

    if not args.quiet:
        logging.basicConfig(stream=sys.stdout, level=logging.INFO)

    exit_gracefully_if_config_file_does_not_exist(args.config_file)

    run_message_handler_for_bot(
        lib_module=lib_module,
        config_file=args.config_file,
        quiet=args.quiet,
        bot_name=bot_name
    )

if __name__ == '__main__':
    main()



# See readme.md for instructions on running this code.


class HelloWorldHandler(object):
    def usage(self):
        return '''
        This is a boilerplate bot that responds to a user query with
        "beep boop", which is robot for "Hello World".

        This bot can be used as a template for other, more
        sophisticated, bots.
        '''

    def handle_message(self, message, bot_handler):
        content = 'beep boop'
        bot_handler.send_reply(message, content)

handler_class = HelloWorldHandler



# See readme.md for instructions on running this code.

from __future__ import absolute_import
from __future__ import division

import copy
import importlib
from math import log10, floor

import re
from zulip_bots.bots.converter import utils

def is_float(value):
    try:
        float(value)
        return True
    except ValueError:
        return False

# Rounds the number 'x' to 'digits' significant digits.
# A normal 'round()' would round the number to an absolute amount of
# fractional decimals, e.g. 0.00045 would become 0.0.
# 'round_to()' rounds only the digits that are not 0.
# 0.00045 would then become 0.0005.
def round_to(x, digits):
    return round(x, digits-int(floor(log10(abs(x)))))

class ConverterHandler(object):
    '''
    This plugin allows users to make conversions between various units,
    e.g. Celsius to Fahrenheit, or kilobytes to gigabytes.
    It looks for messages of the format
    '@mention-bot <number> <unit_from> <unit_to>'
    The message '@mention-bot help' posts a short description of how to use
    the plugin, along with a list of all supported units.
    '''

    def usage(self):
        return '''
               This plugin allows users to make conversions between
               various units, e.g. Celsius to Fahrenheit,
               or kilobytes to gigabytes. It looks for messages of
               the format '@mention-bot <number> <unit_from> <unit_to>'
               The message '@mention-bot help' posts a short description of
               how to use the plugin, along with a list of
               all supported units.
               '''

    def handle_message(self, message, bot_handler):
        bot_response = get_bot_converter_response(message, bot_handler)
        bot_handler.send_reply(message, bot_response)

def get_bot_converter_response(message, bot_handler):
    content = message['content']

    words = content.lower().split()
    convert_indexes = [i for i, word in enumerate(words) if word == "@convert"]
    convert_indexes = [-1] + convert_indexes
    results = []

    for convert_index in convert_indexes:
        if (convert_index + 1) < len(words) and words[convert_index + 1] == 'help':
            results.append(utils.HELP_MESSAGE)
            continue
        if (convert_index + 3) < len(words):
            number = words[convert_index + 1]
            unit_from = utils.ALIASES.get(words[convert_index + 2], words[convert_index + 2])
            unit_to = utils.ALIASES.get(words[convert_index + 3], words[convert_index + 3])
            exponent = 0

            if not is_float(number):
                results.append('`' + number + '` is not a valid number. ' + utils.QUICK_HELP)
                continue

            number = float(number)
            number_res = copy.copy(number)

            for key, exp in utils.PREFIXES.items():
                if unit_from.startswith(key):
                    exponent += exp
                    unit_from = unit_from[len(key):]
                if unit_to.startswith(key):
                    exponent -= exp
                    unit_to = unit_to[len(key):]

            uf_to_std = utils.UNITS.get(unit_from, False)
            ut_to_std = utils.UNITS.get(unit_to, False)

            if uf_to_std is False:
                results.append('`' + unit_from + '` is not a valid unit. ' + utils.QUICK_HELP)
            if ut_to_std is False:
                results.append('`' + unit_to + '` is not a valid unit.' + utils.QUICK_HELP)
            if uf_to_std is False or ut_to_std is False:
                continue

            base_unit = uf_to_std[2]
            if uf_to_std[2] != ut_to_std[2]:
                unit_from = unit_from.capitalize() if uf_to_std[2] == 'kelvin' else unit_from
                results.append('`' + unit_to.capitalize() + '` and `' + unit_from + '`' +
                               ' are not from the same category. ' + utils.QUICK_HELP)
                continue

            # perform the conversion between the units
            number_res *= uf_to_std[1]
            number_res += uf_to_std[0]
            number_res -= ut_to_std[0]
            number_res /= ut_to_std[1]

            if base_unit == 'bit':
                number_res *= 1024 ** (exponent // 3)
            else:
                number_res *= 10 ** exponent
            number_res = round_to(number_res, 7)

            results.append('{} {} = {} {}'.format(number,
                                                  words[convert_index + 2],
                                                  number_res,
                                                  words[convert_index + 3]))

        else:
            results.append('Too few arguments given. ' + utils.QUICK_HELP)

    new_content = ''
    for idx, result in enumerate(results, 1):
        new_content += ((str(idx) + '. conversion: ') if len(results) > 1 else '') + result + '\n'

    return new_content

handler_class = ConverterHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from six.moves import zip

from zulip_bots.test_lib import BotTestCase

class TestHelpBot(BotTestCase):
    bot_name = "help"

    def test_bot(self):
        txt = "Info on Zulip can be found here:\nhttps://github.com/zulip/zulip"
        messages = ["", "help", "Hi, my name is abc"]
        self.check_expected_responses(list(zip(messages, len(messages)*[txt])))



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

import mock
from mock import MagicMock, patch
from zulip_bots.test_lib import BotTestCase

class TestXkcdBot(BotTestCase):
    bot_name = "xkcd"

    @mock.patch('logging.exception')
    def test_bot(self, mock_logging_exception):
        help_txt = "xkcd bot supports these commands:"
        err_txt  = "xkcd bot only supports these commands, not `{}`:"
        commands = '''
* `@xkcd help` to show this help message.
* `@xkcd latest` to fetch the latest comic strip from xkcd.
* `@xkcd random` to fetch a random comic strip from xkcd.
* `@xkcd <comic id>` to fetch a comic strip based on `<comic id>` e.g `@xkcd 1234`.'''
        invalid_id_txt = "Sorry, there is likely no xkcd comic strip with id: #"

        # 'latest' query
        bot_response = ("#1866: **Russell's Teapot**\n"
                        "[Unfortunately, NASA regulations state that Bertrand Russell-related "
                        "payloads can only be launched within launch vehicles which do not launch "
                        "themselves.](https://imgs.xkcd.com/comics/russells_teapot.png)")
        with self.mock_http_conversation('test_latest'):
            self.assert_bot_response(
                message = {'content': 'latest'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # 'random' query
        bot_response = ("#1800: **Chess Notation**\n"
                        "[I've decided to score all my conversations using chess win-loss "
                        "notation. (??)](https://imgs.xkcd.com/comics/chess_notation.png)")
        with self.mock_http_conversation('test_random'):
            # Mock randint function.
            with patch('zulip_bots.bots.xkcd.xkcd.random.randint') as randint:
                mock_rand_value = mock.MagicMock()
                mock_rand_value.return_value = 1800
                randint.return_value = mock_rand_value.return_value

                self.assert_bot_response(
                    message = {'content': 'random'},
                    response = {'content': bot_response},
                    expected_method='send_reply'
                )

        # specific comic ID query
        bot_response = ("#1: **Barrel - Part 1**\n[Don't we all.]"
                        "(https://imgs.xkcd.com/comics/barrel_cropped_(1).jpg)")
        with self.mock_http_conversation('test_specific_id'):
            self.assert_bot_response(
                message = {'content': '1'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Comic ID doesn't exist.
        bot_response = invalid_id_txt + "999999999"
        with self.mock_http_conversation('test_not_existing_id'):
            self.assert_bot_response(
                message = {'content': '999999999'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        bot_response = invalid_id_txt + "0"
        with self.mock_http_conversation('test_not_existing_id_2'):
            self.assert_bot_response(
                message = {'content': '0'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Empty query, no request made to the Internet.
        bot_response = err_txt.format('')+commands
        self.assert_bot_response(
            message = {'content': ''},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # 'help' command.
        bot_response = help_txt+commands
        self.assert_bot_response(
            message = {'content': 'help'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # wrong command.
        bot_response = err_txt.format('x')+commands
        self.assert_bot_response(
            message = {'content': 'x'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestWeatherBot(BotTestCase):
    bot_name = "weather"

    def test_bot(self):

        # City query
        bot_response = "Weather in New York, US:\n71.33 F / 21.85 C\nMist"
        with self.mock_config_info({'key': '123456'}), \
                self.mock_http_conversation('test_only_city'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'New York'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # City with country query
        bot_response = "Weather in New Delhi, IN:\n80.33 F / 26.85 C\nMist"
        with self.mock_config_info({'key': '123456'}), \
                self.mock_http_conversation('test_city_with_country'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'New Delhi, India'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # Only country query: returns the weather of the capital city
        bot_response = "Weather in London, GB:\n58.73 F / 14.85 C\nShower Rain"
        with self.mock_config_info({'key': '123456'}), \
                self.mock_http_conversation('test_only_country'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'United Kingdom'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

        # City not found query
        bot_response = "Sorry, city not found"
        with self.mock_config_info({'key': '123456'}), \
                self.mock_http_conversation('test_city_not_found'):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'fghjklasdfgh'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )
        help_content = '''
            This bot returns weather info for specified city.
            You specify city in the following format:
            city, state/country
            state and country parameter is optional(useful when there are many cities with the same name)
            For example:
            @**Weather Bot** Portland
            @**Weather Bot** Portland, Me
            '''.strip()

        # help message
        bot_response = help_content
        self.assert_bot_response(
            message = {'content': 'help'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

        # empty message
        bot_response = help_content
        self.assert_bot_response(
            message = {'content': ''},
            response = {'content': bot_response},
            expected_method='send_reply'
        )



# See readme.md for instructions on running this code.
import logging
import json
import requests
import html2text
import string

class DefineHandler(object):
    '''
    This plugin define a word that the user inputs. It
    looks for messages starting with '@mention-bot'.
    '''

    DEFINITION_API_URL = 'https://owlbot.info/api/v1/dictionary/{}?format=json'
    REQUEST_ERROR_MESSAGE = 'Could not load definition.'
    EMPTY_WORD_REQUEST_ERROR_MESSAGE = 'Please enter a word to define.'
    PHRASE_ERROR_MESSAGE = 'Definitions for phrases are not available.'
    SYMBOLS_PRESENT_ERROR_MESSAGE = 'Definitions of words with symbols are not possible.'

    def usage(self):
        return '''
            This plugin will allow users to define a word. Users should preface
            messages with @mention-bot.
            '''

    def handle_message(self, message, bot_handler):
        original_content = message['content'].strip()
        bot_response = self.get_bot_define_response(original_content)

        bot_handler.send_reply(message, bot_response)

    def get_bot_define_response(self, original_content):
        split_content = original_content.split(' ')
        # If there are more than one word (a phrase)
        if len(split_content) > 1:
            return DefineHandler.PHRASE_ERROR_MESSAGE

        to_define = split_content[0].strip()
        to_define_lower = to_define.lower()

        # Check for presence of non-letters
        non_letters = set(to_define_lower) - set(string.ascii_lowercase)
        if len(non_letters):
            return self.SYMBOLS_PRESENT_ERROR_MESSAGE

        # No word was entered.
        if not to_define_lower:
            return self.EMPTY_WORD_REQUEST_ERROR_MESSAGE
        else:
            response = '**{}**:\n'.format(to_define)

            try:
                # Use OwlBot API to fetch definition.
                api_result = requests.get(self.DEFINITION_API_URL.format(to_define_lower))
                # Convert API result from string to JSON format.
                definitions = api_result.json()

                # Could not fetch definitions for the given word.
                if not definitions:
                    response += self.REQUEST_ERROR_MESSAGE
                else:  # Definitions available.
                    # Show definitions line by line.
                    for d in definitions:
                        example = d['example'] if d['example'] else '*No example available.*'
                        response += '\n' + '* (**{}**) {}\n&nbsp;&nbsp;{}'.format(d['type'], d['defenition'], html2text.html2text(example))

            except Exception as e:
                response += self.REQUEST_ERROR_MESSAGE
                logging.exception(e)

            return response

handler_class = DefineHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

import json

from zulip_bots.test_lib import BotTestCaseBase

class TestGithubDetailBot(BotTestCaseBase):
    bot_name = "github_detail"
    mock_config = {'owner': 'zulip', 'repo': 'zulip'}

    # Overrides default test_bot_usage().
    def test_bot_usage(self):
        # type: () -> None
        with self.mock_config_info(self.mock_config):
            self.initialize_bot()
            self.assertNotEqual(self.message_handler.usage(), '')

    def test_issue(self):
        bot_response = '**[zulip/zulip#5365](https://github.com/zulip/zulip/issues/5365)'\
                       ' - frontend: Enable hot-reloading of CSS in development**\n'\
                       'Created by **[timabbott](https://github.com/timabbott)**\n'\
                       'Status - **Open**\n'\
                       '```quote\n'\
                       'There\'s strong interest among folks working on the frontend in being '\
                       'able to use the hot-reloading feature of webpack for managing our CSS.\r\n\r\n'\
                       'In order to do this, step 1 is to move our CSS minification pipeline '\
                       'from django-pipeline to Webpack.  \n```'
        # This message calls the `send_reply` function of BotHandlerApi
        with self.mock_http_conversation('test_issue'):
            self.assert_bot_response(
                message = {'content': 'zulip/zulip#5365'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

    def test_pull_request(self):
        bot_response = '**[zulip/zulip#5345](https://github.com/zulip/zulip/pull/5345)'\
                       ' - [WIP] modal: Replace bootstrap modal with custom modal class**\n'\
                       'Created by **[jackrzhang](https://github.com/jackrzhang)**\n'\
                       'Status - **Open**\n```quote\nAn interaction bug (#4811)  '\
                       'between our settings UI and the bootstrap modals breaks hotkey '\
                       'support for `Esc` when multiple modals are open.\r\n\r\ntodo:\r\n[x]'\
                       ' Create `Modal` class in `modal.js` (drafted by @brockwhittaker)\r\n[x]'\
                       ' Reimplement change_email_modal utilizing `Modal` class\r\n[] Dump '\
                       'using bootstrap for the account settings modal and all other modals,'\
                       ' replace with `Modal` class\r\n[] Add hotkey support for closing the'\
                       ' top modal for `Esc`\r\n\r\nThis should also be a helpful step in removing dependencies from Bootstrap.\n```'
        # This message calls the `send_reply` function of BotHandlerApi
        with self.mock_http_conversation('test_pull'):
            self.assert_bot_response(
                message = {'content': 'zulip/zulip#5345'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

    def test_404(self):
        bot_response = 'Failed to find issue/pr: zulip/zulip#0'
        # This message calls the `send_reply` function of BotHandlerApi
        with self.mock_http_conversation('test_404'):
            self.assert_bot_response(
                message = {'content': 'zulip/zulip#0'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )

    def test_random_text(self):
        bot_response = 'Failed to find any issue or PR.'
        # This message calls the `send_reply` function of BotHandlerApi
        self.assert_bot_response(
            message = {'content': 'some random text'},
            response = {'content': bot_response},
            expected_method='send_reply'
        )

    def test_help_text(self):
        bot_response = 'This plugin displays details on github issues and pull requests. '\
                       'To reference an issue or pull request usename mention the bot then '\
                       'anytime in the message type its id, for example:\n@**Github detail** '\
                       '#3212 zulip#3212 zulip/zulip#3212\nThe default owner is zulip and '\
                       'the default repo is zulip.'
        # This message calls the `send_reply` function of BotHandlerApi

        with self.mock_config_info(self.mock_config):
            self.initialize_bot()
            self.assert_bot_response(
                message = {'content': 'help'},
                response = {'content': bot_response},
                expected_method='send_reply'
            )



# See readme.md for instructions on running this code.


class IncrementorHandler(object):
    META = {
        'name': 'Incrementor',
        'description': 'Example bot to test the update_message() function.',
    }

    def usage(self):
        return '''
        This is a boilerplate bot that makes use of the
        update_message function. For the first @-mention, it initially
        replies with one message containing a `1`. Every time the bot
        is @-mentioned, this number will be incremented in the same message.
        '''

    def initialize(self, bot_handler):
        storage = bot_handler.storage
        if not storage.contains('number') or not storage.contains('message_id'):
            storage.put('number', 0)
            storage.put('message_id', None)

    def handle_message(self, message, bot_handler):
        storage = bot_handler.storage
        num = storage.get('number')
        storage.put('number', num + 1)
        if storage.get('message_id') is None:
            result = bot_handler.send_reply(message, str(storage.get('number')))
            storage.put('message_id', result['id'])
        else:
            bot_handler.update_message(dict(
                message_id = storage.get('message_id'),
                content = str(storage.get('number'))
            ))


handler_class = IncrementorHandler



import random

import logging
import requests

XKCD_TEMPLATE_URL = 'https://xkcd.com/%s/info.0.json'
LATEST_XKCD_URL = 'https://xkcd.com/info.0.json'

class XkcdHandler(object):
    '''
    This plugin provides several commands that can be used for fetch a comic
    strip from https://xkcd.com. The bot looks for messages starting with
    "@mention-bot" and responds with a message with the comic based on provided
    commands.
    '''

    META = {
        'name': 'XKCD',
        'description': 'Fetches comic strips from https://xkcd.com.',
    }

    def usage(self):
        return '''
            This plugin allows users to fetch a comic strip provided by
            https://xkcd.com. Users should preface the command with "@mention-bot".

            There are several commands to use this bot:
            - @mention-bot help -> To show all commands the bot supports.
            - @mention-bot latest -> To fetch the latest comic strip from xkcd.
            - @mention-bot random -> To fetch a random comic strip from xkcd.
            - @mention-bot <comic_id> -> To fetch a comic strip based on
            `<comic_id>`, e.g `@mention-bot 1234`.
            '''

    def handle_message(self, message, bot_handler):
        xkcd_bot_response = get_xkcd_bot_response(message)
        bot_handler.send_reply(message, xkcd_bot_response)

class XkcdBotCommand(object):
    LATEST = 0
    RANDOM = 1
    COMIC_ID = 2

class XkcdNotFoundError(Exception):
    pass

class XkcdServerError(Exception):
    pass

def get_xkcd_bot_response(message):
    original_content = message['content'].strip()
    command = original_content.strip()

    commands_help = ("%s"
                     "\n* `@xkcd help` to show this help message."
                     "\n* `@xkcd latest` to fetch the latest comic strip from xkcd."
                     "\n* `@xkcd random` to fetch a random comic strip from xkcd."
                     "\n* `@xkcd <comic id>` to fetch a comic strip based on `<comic id>` "
                     "e.g `@xkcd 1234`.")

    try:
        if command == 'help':
            return commands_help % ('xkcd bot supports these commands:')
        elif command == 'latest':
            fetched = fetch_xkcd_query(XkcdBotCommand.LATEST)
        elif command == 'random':
            fetched = fetch_xkcd_query(XkcdBotCommand.RANDOM)
        elif command.isdigit():
            fetched = fetch_xkcd_query(XkcdBotCommand.COMIC_ID, command)
        else:
            return commands_help % ("xkcd bot only supports these commands, not `%s`:" % (command,))
    except (requests.exceptions.ConnectionError, XkcdServerError):
        logging.exception('Connection error occurred when trying to connect to xkcd server')
        return 'Sorry, I cannot process your request right now, please try again later!'
    except XkcdNotFoundError:
        logging.exception('XKCD server responded 404 when trying to fetch comic with id %s'
                          % (command))
        return 'Sorry, there is likely no xkcd comic strip with id: #%s' % (command,)
    else:
        return ("#%s: **%s**\n[%s](%s)" % (fetched['num'],
                                           fetched['title'],
                                           fetched['alt'],
                                           fetched['img']))

def fetch_xkcd_query(mode, comic_id=None):
    try:
        if mode == XkcdBotCommand.LATEST:  # Fetch the latest comic strip.
            url = LATEST_XKCD_URL

        elif mode == XkcdBotCommand.RANDOM:  # Fetch a random comic strip.
            latest = requests.get(LATEST_XKCD_URL)

            if latest.status_code != 200:
                raise XkcdServerError()

            latest_id = latest.json()['num']
            random_id = random.randint(1, latest_id)
            url = XKCD_TEMPLATE_URL % (str(random_id))

        elif mode == XkcdBotCommand.COMIC_ID:  # Fetch specific comic strip by id number.
            if comic_id is None:
                raise Exception('Missing comic_id argument')
            url = XKCD_TEMPLATE_URL % (comic_id)

        fetched = requests.get(url)

        if fetched.status_code == 404:
            raise XkcdNotFoundError()
        elif fetched.status_code != 200:
            raise XkcdServerError()

        xkcd_json = fetched.json()
    except requests.exceptions.ConnectionError as e:
        logging.warning(e)
        raise

    return xkcd_json

handler_class = XkcdHandler



import re
import os
import sys
import logging
import six.moves.configparser

import requests

class GithubHandler(object):
    '''
    This bot provides details on github issues and pull requests when they're
    referenced in the chat.
    '''

    GITHUB_ISSUE_URL_TEMPLATE = 'https://api.github.com/repos/{owner}/{repo}/issues/{id}'
    HANDLE_MESSAGE_REGEX = re.compile("(?:([\w-]+)\/)?([\w-]+)?#(\d+)")

    def initialize(self, bot_handler):
        self.config_info = bot_handler.get_config_info('github_detail', optional=True)
        self.owner = self.config_info.get("owner", False)
        self.repo = self.config_info.get("repo", False)

    def usage(self):
        # type: () -> None
        return ("This plugin displays details on github issues and pull requests. "
                "To reference an issue or pull request usename mention the bot then "
                "anytime in the message type its id, for example:\n"
                "@**Github detail** #3212 zulip#3212 zulip/zulip#3212\n"
                "The default owner is {} and the default repo is {}.".format(self.owner, self.repo))

    def format_message(self, details):
        # type: (Dict[Text, Union[Text, int, bool]]) -> Text
        number = details['number']
        title = details['title']
        link = details['html_url']
        author = details['user']['login']
        owner = details['owner']
        repo = details['repo']

        description = details['body']
        status = details['state'].title()

        message_string = ('**[{owner}/{repo}#{id}]'.format(owner=owner, repo=repo, id=number),
                          '({link}) - {title}**\n'.format(title=title, link=link),
                          'Created by **[{author}](https://github.com/{author})**\n'.format(author=author),
                          'Status - **{status}**\n```quote\n{description}\n```'.format(status=status, description=description))
        return ''.join(message_string)

    def get_details_from_github(self, owner, repo, number):
        # type: (Text, Text, Text) -> Dict[Text, Union[Text, Int, Bool]]
        # Gets the details of an issues or pull request

        try:
            r = requests.get(
                self.GITHUB_ISSUE_URL_TEMPLATE.format(owner=owner, repo=repo, id=number))
        except requests.exceptions.RequestException as e:
            logging.exception(e)
            return
        if r.status_code != requests.codes.ok:
            return
        return r.json()

    def get_owner_and_repo(self, issue_pr):
        owner = issue_pr.group(1)
        repo = issue_pr.group(2)
        if owner is None:
            owner = self.owner
            if repo is None:
                repo = self.repo
        return (owner, repo)

    def handle_message(self, message, bot_handler):
        # type: () -> None
        # Send help message
        if message['content'] == 'help':
            bot_handler.send_reply(message, self.usage())
            return

        # Capture owner, repo, id
        issue_prs = re.finditer(
            self.HANDLE_MESSAGE_REGEX, message['content'])
        bot_messages = []
        for issue_pr in issue_prs:
            owner, repo = self.get_owner_and_repo(issue_pr)
            if owner and repo:
                details = self.get_details_from_github(owner, repo, issue_pr.group(3))
                if details is not None:
                    details['owner'] = owner
                    details['repo'] = repo
                    bot_messages.append(self.format_message(details))
                else:
                    bot_messages.append("Failed to find issue/pr: {owner}/{repo}#{id}"
                                        .format(owner=owner, repo=repo, id=issue_pr.group(3)))
            else:
                bot_messages.append("Failed to detect owner and repository name.")
        if len(bot_messages) == 0:
            bot_messages.append("Failed to find any issue or PR.")
        bot_message = '\n'.join(bot_messages)
        bot_handler.send_reply(message, bot_message)

handler_class = GithubHandler



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import os
import sys
if False:
    from typing import Any, Dict, Optional

import generate_manifest

ZULIP_BOTS_VERSION = "0.3.8"
IS_PYPA_PACKAGE = True

# IS_PYPA_PACKAGE is set to True by tools/release-packages
# before making a PyPA release.
if not IS_PYPA_PACKAGE:
    generate_manifest.generate_dev_manifest()

# We should be installable with either setuptools or distutils.
package_info = dict(
    name='zulip_bots',
    version=ZULIP_BOTS_VERSION,
    description='Zulip\'s Bot framework',
    author='Zulip Open Source Project',
    author_email='zulip-devel@googlegroups.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Communications :: Chat',
    ],
    url='https://www.zulip.org/',
    entry_points={
        'console_scripts': [
            'zulip-run-bot=zulip_bots.run:main',
            'zulip-bot-output=zulip_bots.zulip_bot_output:main'
        ],
    },
    include_package_data=True,
    cmdclass={
        'gen_manifest': generate_manifest.GenerateManifest,
    },
)  # type: Dict[str, Any]

setuptools_info = dict(
    install_requires=[
        'zulip',
        'mock>=2.0.0',
        'html2text',  # for bots/define
    ],
)

try:
    from setuptools import setup, find_packages
    package_info.update(setuptools_info)
    package_info['packages'] = find_packages()

except ImportError:
    from distutils.core import setup
    from distutils.version import LooseVersion
    from importlib import import_module

    # Manual dependency check
    def check_dependency_manually(module_name, version=None):
        # type: (str, Optional[str]) -> None
        try:
            module = import_module(module_name)  # type: Any
            if version is not None:
                assert(LooseVersion(module.__version__) >= LooseVersion(version))
        except (ImportError, AssertionError):
            if version is not None:
                print("{name}>={version} is not installed.".format(
                    name=module_name, version=version), file=sys.stderr)
            else:
                print("{name} is not installed.".format(name=module_name), file=sys.stderr)
            sys.exit(1)

    check_dependency_manually('zulip')
    check_dependency_manually('mock', '2.0.0')
    check_dependency_manually('html2text')
    check_dependency_manually('PyDictionary')

    # Include all submodules under bots/
    package_list = ['zulip_bots']
    dirs = os.listdir('zulip_bots/bots/')
    for dir_name in dirs:
        if os.path.isdir(os.path.join('zulip_bots/bots/', dir_name)):
            package_list.append('zulip_bots.bots.' + dir_name)
    package_info['packages'] = package_list


setup(**package_info)



#!/usr/bin/env python

from __future__ import absolute_import

import argparse
import logging
import os
import sys
import glob
import pip
from typing import Iterator

def provision_bot(path_to_bot, force):
    # type: (str, bool) -> None
    req_path = os.path.join(path_to_bot, 'requirements.txt')
    install_path = os.path.join(path_to_bot, 'bot_dependencies')

    if os.path.isfile(req_path):
        bot_name = os.path.basename(path_to_bot)
        logging.info('Installing dependencies for {}...'.format(bot_name))
        if not os.path.isdir(install_path):
            os.makedirs(install_path)

        # pip install -r $BASEDIR/requirements.txt -t $BASEDIR/bot_dependencies --quiet
        rcode = pip.main(['install', '-r', req_path, '-t', install_path, '--quiet'])

        if rcode != 0:
            logging.error('Error. Check output of `pip install` above for details.')
            if not force:
                logging.error('Use --force to try running anyway.')
                sys.exit(rcode)  # Use pip's exit code
        else:
            logging.info('Installed dependencies successfully.')

        sys.path.insert(0, install_path)


def parse_args(available_bots):
    # type: (Iterator[str]) -> argparse.Namespace
    usage = """
Installs dependencies of bots in the bots/<bot_name>
directories. Add a requirements.txt file in a bot's folder
before provisioning.

To provision all bots, use:
./provision.py

To provision specific bots, use:
./provision.py [names of bots]
Example: ./provision.py helloworld xkcd wikipedia
"""
    parser = argparse.ArgumentParser(usage=usage)

    parser.add_argument('bots_to_provision',
                        metavar='bots',
                        nargs='*',
                        default=available_bots,
                        help='specific bots to provision (default is all)')

    parser.add_argument('--force',
                        default=False,
                        action="store_true",
                        help='Continue installation despite pip errors.')

    parser.add_argument('--quiet', '-q',
                        action='store_true',
                        default=False,
                        help='Turn off logging output.')

    return parser.parse_args()


def main():
    # type: () -> None
    current_dir = os.path.dirname(os.path.abspath(__file__))
    bots_dir = os.path.join(current_dir, "bots")
    bots_subdirs = map(lambda d: os.path.abspath(d), glob.glob(bots_dir + '/*'))
    available_bots = filter(lambda d: os.path.isdir(d), bots_subdirs)

    options = parse_args(available_bots)

    if not options.quiet:
        logging.basicConfig(stream=sys.stdout, level=logging.INFO)

    for bot in options.bots_to_provision:
        provision_bot(os.path.join(bots_dir, bot), options.force)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

import os

import json
import logging
import mock
import requests
import unittest

from mock import MagicMock, patch

from zulip_bots.lib import StateHandler
import zulip_bots.lib
from six.moves import zip

from contextlib import contextmanager
from importlib import import_module
from unittest import TestCase

from typing import List, Dict, Any, Optional, Callable, Tuple
from types import ModuleType

from copy import deepcopy

class BotTestCaseBase(TestCase):
    """Test class for common Bot test helper methods"""
    bot_name = ''  # type: str

    def get_bot_message_handler(self):
        # type: () -> Any
        # message_handler is of type 'Any', since it can contain any bot's
        # handler class. Eventually, we want bot's handler classes to
        # inherit from a common prototype specifying the handle_message
        # function.
        lib_module = import_module('zulip_bots.bots.{bot}.{bot}'.format(bot=self.bot_name))  # type: Any
        return lib_module.handler_class()

    def setUp(self):
        # type: () -> None
        # Mocking ExternalBotHandler
        self.patcher = patch('zulip_bots.lib.ExternalBotHandler', autospec=True)
        self.MockClass = self.patcher.start()
        self.mock_bot_handler = self.MockClass(None, None)
        self.mock_client = MagicMock()
        self.mock_client.get_storage.return_value = {'result': 'success', 'state': {}}
        self.mock_client.update_storage.return_value = {'result': 'success'}
        self.mock_bot_handler.storage = StateHandler(self.mock_client)
        self.mock_bot_handler.send_message.return_value = {'id': 42}
        self.mock_bot_handler.send_reply.return_value = {'id': 42}
        self.message_handler = self.get_bot_message_handler()

    def tearDown(self):
        # type: () -> None
        self.patcher.stop()

    def initialize_bot(self):
        # type: () -> None
        self.message_handler.initialize(self.mock_bot_handler)

    def check_expected_responses(self, expectations, expected_method='send_reply',
                                 email="foo_sender@zulip.com", recipient="foo", subject="foo",
                                 sender_id=0, sender_full_name="Foo Bar", type="all"):
        # type: (List[Tuple[str, Any]], str, str, str, str, int, str, str) -> None
        # To test send_message, Any would be a Dict type,
        # to test send_reply, Any would be a str type.
        if type not in ["private", "stream", "all"]:
            logging.exception("check_expected_response expects type to be 'private', 'stream' or 'all'")

        private_message_template = {'type': "private", 'display_recipient': recipient,
                                    'sender_email': email, 'sender_id': sender_id,
                                    'sender_full_name': sender_full_name}
        stream_message_template = {'type': "stream", 'display_recipient': recipient,
                                   'subject': subject, 'sender_email': email, 'sender_id': sender_id,
                                   'sender_full_name': sender_full_name}

        message_templates = []
        if type in ["private", "all"]:
            message_templates.append(private_message_template)
        if type in ["stream", "all"]:
            message_templates.append(stream_message_template)

        initial_storage = deepcopy(self.mock_bot_handler.storage)
        for message_template in message_templates:
            # A new copy of the StateHandler is used for every new conversation with a
            # different base template. This avoids type="all" failing if the created state
            # of a prior conversation influences the current one.
            self.mock_bot_handler.storage = deepcopy(initial_storage)
            for m, r in expectations:
                # For calls with send_reply, r is a string (the content of a message),
                # so we need to add it to a Dict as the value of 'content'.
                # For calls with send_message, r is already a Dict.
                message = dict(message_template, content = m)
                response = {'content': r} if expected_method == 'send_reply' else r
                self.assert_bot_response(message=message, response=response,
                                         expected_method=expected_method)

    def call_request(self, message, expected_method, response):
        # type: (Dict[str, Any], str, Dict[str, Any]) -> None
        # Send message to the concerned bot
        self.message_handler.handle_message(message, self.mock_bot_handler)

        # Check if the bot is sending a message via `send_message` function.
        # Where response is a dictionary here.
        if expected_method == "send_message":
            self.mock_bot_handler.send_message.assert_called_with(response)
        else:
            self.mock_bot_handler.send_reply.assert_called_with(message, response['content'])

    @contextmanager
    def mock_config_info(self, config_info):
        # type: (Dict[str, str]) -> Any
        self.mock_bot_handler.get_config_info.return_value = config_info
        yield
        self.mock_bot_handler.get_config_info.return_value = None

    @contextmanager
    def mock_http_conversation(self, test_name):
        # type: (str) -> Any
        """
        Use this context manager to mock and verify a bot's HTTP
        requests to the third-party API (and provide the correct
        third-party API response. This allows us to test things
        that would require the Internet without it).
        """
        assert test_name is not None
        base_path = os.path.realpath(os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'bots', self.bot_name, 'fixtures'))
        http_data_path = os.path.join(base_path, '{}.json'.format(test_name))
        with open(http_data_path, 'r') as http_data_file:
            http_data = json.load(http_data_file)
            http_request = http_data.get('request')
            http_response = http_data.get('response')
            http_headers = http_data.get('response-headers')
            with patch('requests.get') as mock_get:
                mock_result = mock.MagicMock()
                mock_result.json.return_value = http_response
                if 'text' in http_response:
                    mock_result.text = http_response.get('text', None)
                mock_result.status_code = http_headers.get('status', 200)
                mock_result.ok.return_value = http_headers.get('ok', True)
                mock_get.return_value = mock_result
                yield
                if 'params' in http_request:
                    params = http_request.get('params', None)
                    mock_get.assert_called_with(http_request['api_url'], params=params)
                elif 'headers' in http_request:
                    headers = http_request.get('headers', None)
                    mock_get.assert_called_with(http_request['api_url'], headers=headers)
                else:
                    mock_get.assert_called_with(http_request['api_url'])

    def assert_bot_response(self, message, response, expected_method):
        # type: (Dict[str, Any], Dict[str, Any], str) -> None
        # Strictly speaking, this function is not needed anymore,
        # kept for now for legacy reasons.
        self.call_request(message, expected_method, response)

class BotTestCase(BotTestCaseBase):
    """Test class extending BotTestCaseBase to add common default tests
    that should be run (by default) for all our bots"""
    def test_bot_usage(self):
        # type: () -> None
        self.assertNotEqual(self.message_handler.usage(), '')



from __future__ import print_function

import json
import logging
import os
import signal
import sys
import time
import re

from six.moves import configparser

from contextlib import contextmanager

if False:
    from mypy_extensions import NoReturn
from typing import Any, Optional, List, Dict, IO, Text, Set
from types import ModuleType

from zulip import Client, ZulipError

def exit_gracefully(signum, frame):
    # type: (int, Optional[Any]) -> None
    sys.exit(0)

def get_bots_directory_path():
    # type: () -> str
    current_dir = os.path.dirname(os.path.abspath(__file__))
    return os.path.join(current_dir, 'bots')

class RateLimit(object):
    def __init__(self, message_limit, interval_limit):
        # type: (int, int) -> None
        self.message_limit = message_limit
        self.interval_limit = interval_limit
        self.message_list = []  # type: List[float]
        self.error_message = '-----> !*!*!*MESSAGE RATE LIMIT REACHED, EXITING*!*!*! <-----\n'
        'Is your bot trapped in an infinite loop by reacting to its own messages?'

    def is_legal(self):
        # type: () -> bool
        self.message_list.append(time.time())
        if len(self.message_list) > self.message_limit:
            self.message_list.pop(0)
            time_diff = self.message_list[-1] - self.message_list[0]
            return time_diff >= self.interval_limit
        else:
            return True

    def show_error_and_exit(self):
        # type: () -> NoReturn
        logging.error(self.error_message)
        sys.exit(1)

class StateHandlerError(Exception):
    pass

class StateHandler(object):
    def __init__(self, client):
        # type: (Client) -> None
        self._client = client
        self.marshal = lambda obj: json.dumps(obj)
        self.demarshal = lambda obj: json.loads(obj)
        response = self._client.get_storage()
        if response['result'] == 'success':
            self.state_ = response['state']
        else:
            raise StateHandlerError("Error initializing state: {}".format(str(response)))

    def put(self, key, value):
        # type: (Text, Text) -> None
        self.state_[key] = self.marshal(value)
        response = self._client.update_storage({'state': {key: self.state_[key]}})
        if response['result'] != 'success':
            raise StateHandlerError("Error updating state: {}".format(str(response)))

    def get(self, key):
        # type: (Text) -> Text
        return self.demarshal(self.state_[key])

    def contains(self, key):
        # type: (Text) -> bool
        return key in self.state_

class ExternalBotHandler(object):
    def __init__(self, client, root_dir, bot_details={}):
        # type: (Client, str, Dict[str, Any]) -> None
        # Only expose a subset of our Client's functionality
        try:
            user_profile = client.get_profile()
        except ZulipError as e:
            print('''
                ERROR: {}

                Have you not started the server?
                Or did you mis-specify the URL?
                '''.format(e))
            sys.exit(1)

        if user_profile.get('result') == 'error':
            msg = user_profile.get('msg', 'unknown')
            print('''
                ERROR: {}
                '''.format(msg))
            sys.exit(1)

        self._rate_limit = RateLimit(20, 5)
        self._client = client
        self._root_dir = root_dir
        self.bot_details = bot_details
        self._storage = StateHandler(client)
        try:
            self.user_id = user_profile['user_id']
            self.full_name = user_profile['full_name']
            self.email = user_profile['email']
        except KeyError:
            logging.error('Cannot fetch user profile, make sure you have set'
                          ' up the zuliprc file correctly.')
            sys.exit(1)

    @property
    def storage(self):
        # type: () -> StateHandler
        return self._storage

    def send_message(self, message):
        # type: (Dict[str, Any]) -> Dict[str, Any]
        if self._rate_limit.is_legal():
            return self._client.send_message(message)
        else:
            self._rate_limit.show_error_and_exit()

    def send_reply(self, message, response):
        # type: (Dict[str, Any], str) -> Dict[str, Any]
        if message['type'] == 'private':
            return self.send_message(dict(
                type='private',
                to=[x['email'] for x in message['display_recipient'] if self.email != x['email']],
                content=response,
            ))
        else:
            return self.send_message(dict(
                type='stream',
                to=message['display_recipient'],
                subject=message['subject'],
                content=response,
            ))

    def update_message(self, message):
        # type: (Dict[str, Any]) -> Dict[str, Any]
        if self._rate_limit.is_legal():
            return self._client.update_message(message)
        else:
            self._rate_limit.show_error_and_exit()

    def get_config_info(self, bot_name, optional=False):
        # type: (str, Optional[bool]) -> Dict[str, Any]
        conf_file_path = os.path.realpath(os.path.join(self._root_dir, bot_name + '.conf'))
        config = configparser.ConfigParser()
        try:
            with open(conf_file_path) as conf:
                config.readfp(conf)  # type: ignore # readfp->read_file in python 3, so not in stubs
        except IOError:
            if optional:
                return dict()
            raise
        return dict(config.items(bot_name))

    def open(self, filepath):
        # type: (str) -> IO[str]
        filepath = os.path.normpath(filepath)
        abs_filepath = os.path.join(self._root_dir, filepath)
        if abs_filepath.startswith(self._root_dir):
            return open(abs_filepath)
        else:
            raise PermissionError("Cannot open file \"{}\". Bots may only access "
                                  "files in their local directory.".format(abs_filepath))

def extract_query_without_mention(message, client):
    # type: (Dict[str, Any], ExternalBotHandler) -> str
    """
    If the bot is the first @mention in the message, then this function returns
    the stripped message with the bot's @mention removed.  Otherwise, it returns None.
    """
    mention = '@**' + client.full_name + '**'
    if not message['content'].startswith(mention):
        return None
    return message['content'][len(mention):].lstrip()

def is_private_message_from_another_user(message_dict, current_user_id):
    # type: (Dict[str, Any], int) -> bool
    """
    Checks whether a message dict represents a PM from another user.

    This function is used by the embedded bot system in the
    zulip/zulip project, so refactor with care.  See the comments in
    extract_query_without_mention.
    """
    if message_dict['type'] == 'private':
        return current_user_id != message_dict['sender_id']
    return False

def run_message_handler_for_bot(lib_module, quiet, config_file, bot_name):
    # type: (Any, bool, str, str) -> Any
    #
    # lib_module is of type Any, since it can contain any bot's
    # handler class. Eventually, we want bot's handler classes to
    # inherit from a common prototype specifying the handle_message
    # function.
    #
    # Set default bot_details, then override from class, if provided
    bot_details = {
        'name': bot_name.capitalize(),
        'description': "",
    }
    bot_details.update(getattr(lib_module.handler_class, 'META', {}))
    # Make sure you set up your ~/.zuliprc

    client_name = "Zulip{}Bot".format(bot_name.capitalize())

    try:
        client = Client(config_file=config_file, client=client_name)
    except configparser.Error as e:
        file_contents = open(config_file).read()
        print('\nERROR: {} seems to be broken:\n\n{}'.format(config_file, file_contents))
        print('\nMore details here:\n\n' + str(e) + '\n')
        sys.exit(1)

    bot_dir = os.path.dirname(lib_module.__file__)
    restricted_client = ExternalBotHandler(client, bot_dir, bot_details)

    message_handler = lib_module.handler_class()
    if hasattr(message_handler, 'initialize'):
        message_handler.initialize(bot_handler=restricted_client)

    if not quiet:
        print("Running {} Bot:".format(bot_details['name']))
        if bot_details['description'] != "":
            print("\n\t{}".format(bot_details['description']))
        print(message_handler.usage())

    def handle_message(message, flags):
        # type: (Dict[str, Any], List[str]) -> None
        logging.info('waiting for next message')

        # `mentioned` will be in `flags` if the bot is mentioned at ANY position
        # (not necessarily the first @mention in the message).
        is_mentioned = 'mentioned' in flags
        is_private_message = is_private_message_from_another_user(message, restricted_client.user_id)

        # Strip at-mention botname from the message
        if is_mentioned:
            # message['content'] will be None when the bot's @-mention is not at the beginning.
            # In that case, the message shall not be handled.
            message['content'] = extract_query_without_mention(message=message, client=restricted_client)
            if message['content'] is None:
                return

        if is_private_message or is_mentioned:
            message_handler.handle_message(
                message=message,
                bot_handler=restricted_client
            )

    signal.signal(signal.SIGINT, exit_gracefully)

    logging.info('starting message handling...')

    def event_callback(event):
        # type: (Dict[str, Any]) -> None
        if event['type'] == 'message':
            handle_message(event['message'], event['flags'])

    client.call_on_each_event(event_callback, ['message'])



# To use this plugin, you need to set up the Giphy API key for this bot in
# giphy.conf in this (zulip_bots/bots/giphy/giphy/) directory.

from __future__ import absolute_import
from __future__ import print_function
from six.moves.configparser import SafeConfigParser
import requests
import logging
import sys
import os
import re

GIPHY_TRANSLATE_API = 'http://api.giphy.com/v1/gifs/translate'

class GiphyHandler(object):
    '''
    This plugin posts a GIF in response to the keywords provided by the user.
    Images are provided by Giphy, through the public API.
    The bot looks for messages starting with @mention of the bot
    and responds with a message with the GIF based on provided keywords.
    It also responds to private messages.
    '''
    def usage(self):
        return '''
            This plugin allows users to post GIFs provided by Giphy.
            Users should preface keywords with the Giphy-bot @mention.
            The bot responds also to private messages.
            '''

    def initialize(self, bot_handler):
        global config_info
        config_info = bot_handler.get_config_info('giphy')

    def handle_message(self, message, bot_handler):
        bot_response = get_bot_giphy_response(message, bot_handler)
        bot_handler.send_reply(message, bot_response)


class GiphyNoResultException(Exception):
    pass


def get_url_gif_giphy(keyword, api_key):
    # Return a URL for a Giphy GIF based on keywords given.
    # In case of error, e.g. failure to fetch a GIF URL, it will
    # return a number.
    query = {'s': keyword,
             'api_key': api_key}
    try:
        data = requests.get(GIPHY_TRANSLATE_API, params=query)
    except requests.exceptions.ConnectionError as e:  # Usually triggered by bad connection.
        logging.warning(e)
        raise

    search_status = data.json()['meta']['status']
    if search_status != 200 or not data.ok:
        raise requests.exceptions.ConnectionError

    try:
        gif_url = data.json()['data']['images']['original']['url']
    except (TypeError, KeyError):  # Usually triggered by no result in Giphy.
        raise GiphyNoResultException()

    return gif_url


def get_bot_giphy_response(message, bot_handler):
    # Each exception has a specific reply should "gif_url" return a number.
    # The bot will post the appropriate message for the error.
    keyword = message['content']
    try:
        gif_url = get_url_gif_giphy(keyword, config_info['key'])
    except requests.exceptions.ConnectionError:
        return ('Uh oh, sorry :slightly_frowning_face:, I '
                'cannot process your request right now. But, '
                'let\'s try again later! :grin:')
    except GiphyNoResultException:
        return ('Sorry, I don\'t have a GIF for "%s"! '
                ':astonished:' % (keyword))
    return ('[Click to enlarge](%s)'
            '[](/static/images/interactive-bot/giphy/powered-by-giphy.png)'
            % (gif_url))

handler_class = GiphyHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestConverterBot(BotTestCase):
    bot_name = "converter"

    def test_bot(self):
        expected = [
            ("", 'Too few arguments given. Enter `@convert help` '
                 'for help on using the converter.\n'),
            ("foo bar", 'Too few arguments given. Enter `@convert help` '
                        'for help on using the converter.\n'),
            ("2 m cm", "2.0 m = 200.0 cm\n"),
            ("12.0 celsius fahrenheit", "12.0 celsius = 53.600054 fahrenheit\n"),
            ("0.002 kilometer millimile", "0.002 kilometer = 1.2427424 millimile\n"),
            ("3 megabyte kilobit", "3.0 megabyte = 24576.0 kilobit\n"),
        ]
        self.check_expected_responses(expected)



# See readme.md for instructions on running this code.

class FollowupHandler(object):
    '''
    This plugin facilitates creating follow-up tasks when
    you are using Zulip to conduct a virtual meeting.  It
    looks for messages starting with '@mention-bot'.

    In this example, we write follow up items to a special
    Zulip stream called "followup," but this code could
    be adapted to write follow up items to some kind of
    external issue tracker as well.
    '''

    def usage(self):
        return '''
            This plugin will allow users to flag messages
            as being follow-up items.  Users should preface
            messages with "@mention-bot".

            Before running this, make sure to create a stream
            called "followup" that your API user can send to.
            '''

    def handle_message(self, message, bot_handler):
        if message['content'] == '':
            bot_response = "Please specify the message you want to send to followup stream after @mention-bot"
            bot_handler.send_reply(message, bot_response)
        else:
            bot_response = self.get_bot_followup_response(message)
            bot_handler.send_message(dict(
                type='stream',
                to='followup',
                subject=message['sender_email'],
                content=bot_response,
            ))

    def get_bot_followup_response(self, message):
        original_content = message['content']
        original_sender = message['sender_email']
        temp_content = 'from %s: ' % (original_sender,)
        new_content = temp_content + original_content

        return new_content

handler_class = FollowupHandler




from __future__ import absolute_import
from __future__ import print_function
import copy
import random
from six.moves import range

initial_board = [["_", "_", "_"],
                 ["_", "_", "_"],
                 ["_", "_", "_"]]

mode = 'r'  # default, can change for debugging to 'p'
def output_mode(string_to_print, mode):
    if mode == "p":
        print(string_to_print)
    elif mode == "r":
        return string_to_print

# -------------------------------------
class TicTacToeGame(object):
    smarter = True
    # If smarter is True, the computer will do some extra thinking - it'll be harder for the user.

    triplets = [[(0, 0), (0, 1), (0, 2)],  # Row 1
                [(1, 0), (1, 1), (1, 2)],  # Row 2
                [(2, 0), (2, 1), (2, 2)],  # Row 3
                [(0, 0), (1, 0), (2, 0)],  # Column 1
                [(0, 1), (1, 1), (2, 1)],  # Column 2
                [(0, 2), (1, 2), (2, 2)],  # Column 3
                [(0, 0), (1, 1), (2, 2)],  # Diagonal 1
                [(0, 2), (1, 1), (2, 0)]   # Diagonal 2
                ]

    positions = "Coordinates are entered in a (row, column) format. Numbering is from top to bottom and left to right.\n" \
                "Here are the coordinates of each position. (Parentheses and spaces are optional.) \n" \
                "(1, 1)  (1, 2)  (1, 3) \n(2, 1)  (2, 2)  (2, 3) \n(3, 1) (3, 2) (3, 3) \n " \
                "Your move would be one of these. To make a move, type @mention-bot " \
                "followed by a space and the coordinate."

    detailed_help_message = "*Help for Tic-Tac-Toe bot* \n" \
                            "The bot responds to messages starting with @mention-bot.\n" \
                            "**@mention-bot new** will start a new game (but not if you're " \
                            "already in the middle of a game). You must type this first to start playing!\n" \
                            "**@mention-bot help** will return this help function.\n" \
                            "**@mention-bot quit** will quit from the current game.\n" \
                            "**@mention-bot <coordinate>** will make a move at the given coordinate.\n" \
                            "Coordinates are entered in a (row, column) format. Numbering is from " \
                            "top to bottom and left to right. \n" \
                            "Here are the coordinates of each position. (Parentheses and spaces are optional). \n" \
                            "(1, 1)  (1, 2)  (1, 3) \n(2, 1)  (2, 2)  (2, 3) \n(3, 1) (3, 2) (3, 3) \n"

    def __init__(self, board):
        self.board = board

    def display_row(self, row):
        ''' Takes the row passed in as a list and returns it as a string. '''
        row_string = " ".join([e.strip() for e in row])
        return("[ {} ]\n".format(row_string))

    def display_board(self, board):
        ''' Takes the board as a nested list and returns a nice version for the user. '''
        return "".join([self.display_row(r) for r in board])

    def get_value(self, board, position):
        return board[position[0]][position[1]]

    def board_is_full(self, board):
        ''' Determines if the board is full or not. '''
        full = False
        board_state = ""
        for row in board:
            for element in row:
                if element == "_":
                    board_state += "_"
        if "_" not in board_state:
            full = True
        return full

    def win_conditions(self, board, triplets):
        ''' Returns true if all coordinates in a triplet have the same value in them (x or o) and no coordinates
        in the triplet are blank. '''
        won = False
        for triplet in triplets:
            if (self.get_value(board, triplet[0]) == self.get_value(board, triplet[1]) ==
                    self.get_value(board, triplet[2]) != "_"):
                won = True
                break
        return won

    def get_locations_of_char(self, board, char):
        ''' Gets the locations of the board that have char in them. '''
        locations = []
        for row in range(3):
            for col in range(3):
                if board[row][col] == char:
                    locations.append([row, col])
        return locations

    def two_blanks(self, triplet, board):
        ''' Determines which rows/columns/diagonals have two blank spaces and an 'o' already in them. It's more advantageous
        for the computer to move there. This is used when the computer makes its move. '''

        o_found = False
        for position in triplet:
            if self.get_value(board, position) == "o":
                o_found = True
                break

        blanks_list = []
        if o_found:
            for position in triplet:
                if self.get_value(board, position) == "_":
                    blanks_list.append(position)

            if len(blanks_list) == 2:
                return blanks_list

    def computer_move(self, board):
        ''' The computer's logic for making its move. '''
        my_board = copy.deepcopy(board)  # First the board is copied; used later on
        blank_locations = self.get_locations_of_char(my_board, "_")
        x_locations = self.get_locations_of_char(board, "x")  # Gets the locations that already have x's
        corner_locations = [[0, 0], [0, 2], [2, 0], [2, 2]]  # List of the coordinates of the corners of the board
        edge_locations = [[1, 0], [0, 1], [1, 2], [2, 1]]  # List of the coordinates of the edge spaces of the board

        if blank_locations == []:  # If no empty spaces are left, the computer can't move anyway, so it just returns the board.
            return board

        if len(x_locations) == 1:  # This is special logic only used on the first move.
            # If the user played first in the corner or edge, the computer should move in the center.
            if x_locations[0] in corner_locations or x_locations[0] in edge_locations:
                board[1][1] = "o"
            # If user played first in the center, the computer should move in the corner. It doesn't matter which corner.
            else:
                location = random.choice(corner_locations)
                row = location[0]
                col = location[1]
                board[row][col] = "o"
            return board

        # This logic is used on all other moves.
        # First I'll check if the computer can win in the next move. If so, that's where the computer will play.
        # The check is done by replacing the blank locations with o's and seeing if the computer would win in each case.
        for row, col in blank_locations:
            my_board[row][col] = "o"
            if self.win_conditions(my_board, self.triplets):
                board[row][col] = "o"
                return board
            else:
                my_board[row][col] = "_"  # Revert if not winning

        # If the computer can't immediately win, it wants to make sure the user can't win in their next move, so it
        # checks to see if the user needs to be blocked.
        # The check is done by replacing the blank locations with x's and seeing if the user would win in each case.
        for row, col in blank_locations:
            my_board[row][col] = "x"
            if self.win_conditions(my_board, self.triplets):
                board[row][col] = "o"
                return board
            else:
                my_board[row][col] = "_"  # Revert if not winning

        # Assuming nobody will win in their next move, now I'll find the best place for the computer to win.
        for row, col in blank_locations:
            if ('x' not in my_board[row] and my_board[0][col] != 'x' and my_board[1][col] !=
                    'x' and my_board[2][col] != 'x'):
                board[row][col] = 'o'
                return board

        # If no move has been made, choose a random blank location. If smarter is True, the computer will choose a
        # random blank location from a set of better locations to play. These locations are determined by seeing if
        # there are two blanks and an 'o' in each row, column, and diagonal (done in two_blanks).
        # If smarter is False, all blank locations can be chosen.
        if self.smarter:
            blanks = []
            for triplet in self.triplets:
                result = self.two_blanks(triplet, board)
                if result:
                    blanks = blanks + result
            blank_set = set(blanks)
            blank_list = list(blank_set)
            if blank_list == []:
                location = random.choice(blank_locations)
            else:
                location = random.choice(blank_list)
            row = location[0]
            col = location[1]
            board[row][col] = 'o'
            return board

        else:
            location = random.choice(blank_locations)
            row = location[0]
            col = location[1]
            board[row][col] = 'o'
            return board

    def check_validity(self, move):
        ''' Checks the validity of the coordinate input passed in to make sure it's not out-of-bounds (ex. 5, 5) '''
        try:
            split_move = move.split(",")
            row = split_move[0].strip()
            col = split_move[1].strip()
            valid = False
            if row == "1" or row == "2" or row == "3":
                if col == "1" or col == "2" or col == "3":
                    valid = True
        except IndexError:
            valid = False
        return valid

    def sanitize_move(self, move):
        ''' As there are various ways to input a coordinate (with/without parentheses, with/without spaces, etc.) the
        input is stripped to just the numbers before being used in the program. '''
        move = move.replace("(", "")
        move = move.replace(")", "")
        move = move.strip()
        return move

    def tictactoe(self, board, input_string):
        return_string = ""
        move = self.sanitize_move(input_string)

        # Subtraction must be done to convert to the right indices, since computers start numbering at 0.
        row = (int(move[0])) - 1
        column = (int(move[-1])) - 1

        if board[row][column] != "_":
            return_string += output_mode("That space is already filled, sorry!", mode)
            return return_string
        else:
            board[row][column] = "x"

        return_string += self.display_board(board)

        # Check to see if the user won/drew after they made their move. If not, it's the computer's turn.
        if self.win_conditions(board, self.triplets):
            return_string += output_mode("Game over! You've won!", mode)
            return return_string

        if self.board_is_full(board):
            return_string += output_mode("It's a draw! Neither of us was able to win.", mode)
            return return_string

        return_string += output_mode("My turn:\n", mode)
        self.computer_move(board)
        return_string += self.display_board(board)

        # Checks to see if the computer won after it makes its move. (The computer can't draw, so there's no point
        # in checking.) If the computer didn't win, the user gets another turn.
        if self.win_conditions(board, self.triplets):
            return_string += output_mode("Game over! I've won!", mode)
            return return_string

        return_string += output_mode("Your turn! Enter a coordinate or type help.", mode)
        return return_string

# -------------------------------------
flat_initial = sum(initial_board, [])
def first_time(board):
    flat = sum(board, [])
    return flat == flat_initial

class ticTacToeHandler(object):
    '''
    You can play tic-tac-toe in a private message with
    tic-tac-toe bot! Make sure your message starts with
    "@mention-bot".
    '''
    META = {
        'name': 'TicTacToe',
        'description': 'Lets you play Tic-tac-toe against a computer.',
    }

    def usage(self):
        return '''
            You can play tic-tac-toe with the computer now! Make sure your
            message starts with @mention-bot.
            '''

    def handle_message(self, message, bot_handler):
        command_list = message['content']
        command = ""
        for val in command_list:
            command += val
        original_sender = message['sender_email']
        storage = bot_handler.storage
        if not storage.contains(original_sender):
            storage.put(original_sender, None)
        user_board = storage.get(original_sender)
        if (not user_board) and command == "new":
            user_board = copy.deepcopy(initial_board)
            storage.put(original_sender, user_board)
        user_game = TicTacToeGame(user_board) if user_board else None

        if command == 'new':
            if user_game and not first_time(user_game.board):
                return_content = "You're already playing a game! Type **@tictactoe help** or **@ttt help** to see valid inputs."
            else:
                return_content = "Welcome to tic-tac-toe! You'll be x's and I'll be o's. Your move first!\n"
                return_content += TicTacToeGame.positions
        elif command == 'help':
            return_content = TicTacToeGame.detailed_help_message
        elif (user_game) and TicTacToeGame.check_validity(user_game, TicTacToeGame.sanitize_move(user_game, command)):
            return_content = TicTacToeGame.tictactoe(user_game, user_board, command)
            storage.put(original_sender, user_board)
        elif (user_game) and command == 'quit':
            storage.put(original_sender, None)
            return_content = "You've successfully quit the game."
        else:
            return_content = "Hmm, I didn't understand your input. Type **@tictactoe help** or **@ttt help** to see valid inputs."

        if "Game over" in return_content or "draw" in return_content:
            storage.put(original_sender, None)

        bot_handler.send_message(dict(
            type = 'private',
            to = original_sender,
            subject = message['sender_email'],
            content = return_content,
        ))

handler_class = ticTacToeHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase
from zulip_bots.lib import StateHandler


class TestIncrementorBot(BotTestCase):
    bot_name = "incrementor"

    def test_bot(self):
        self.initialize_bot()
        messages = [  # Template for message inputs to test, absent of message content
            {
                'type': 'stream',
                'display_recipient': 'some stream',
                'subject': 'some subject',
                'sender_email': 'foo_sender@zulip.com',
            },
            {
                'type': 'private',
                'sender_email': 'foo_sender@zulip.com',
            },
        ]
        self.assert_bot_response(dict(messages[0], content=""), {'content': "1"},
                                 'send_reply')
        # Last test commented out since we don't have update_message
        # support in the test framework yet.

        # self.assert_bot_response(dict(messages[0], content=""), {'message_id': 5, 'content': "2"},
        #                          'update_message', storage)
